
/*


 Copyright: 
 2010 daid KAHL, OTA shinsuke, HASHIMOTO takashi, YAMAGUCHI hidetoshi 
 2011, 2012, 2013, 2015 daid KAHL
 2015 KIM Dahee

 This file is modified from crabat

 crabat is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 crabat is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with crabat.  If not, see <http://www.gnu.org/licenses/>.
  
*/

/**
@file Analyzer.cxx
@author daid, Dahee
@version 1.0
@brief Analyzer for the run data.
@details Takes the Chain and loops over all entries.\n
Fill histograms, do physics.
@date 07 Nov 2011 18:57:28 
*/

//////////////////////////////////////
//    analyze the data here!
/////////////////////////////////////
// If run from ROOT, exit with message
#if defined (__CINT__)
{
cout << "This is C++ code, not a ROOT macro!" << endl;
cout << "To continue, run:" << endl;
cout << "make" << endl;
cout << "./run" << endl;
gApplication->Terminate();
}
#endif
//
#define Analyzer_cxx

#include "Analyzer.h" // all other headers are here
#include "Analyzer_config.cxx"
#include "Calibration.cxx"

// hopefully we don't need it!
//#include "libs.


/**
 * @brief Loops over the TChain and entries
 * @details Oh my god there are so many details here.
 */
void Analyzer::Loop(Int_t run,
     Bool_t flag_raw, Bool_t flag_detail,
     Bool_t flag_ssd, Bool_t flag_ppac)
{


  if (fChain == 0) return;


//  Long64_t nentries = fChain->GetEntriesFast(); //GetentriesFast emits wrong entry number
  Long64_t nentries = fChain->GetEntries();
  cout  << "Entries: " << nentries << endl;
  Calibration *psd0_X_calib=new Calibration("psd0_X_calib.dat",16);//
  Calibration *psd0_Y_calib=new Calibration("psd0_Y_calib.dat",16);//
  Calibration *psd1_X_calib=new Calibration("psd1_X_calib.dat",16);//
  Calibration *psd1_Y_calib=new Calibration("psd1_Y_calib.dat",16);//

  Calibration *psd0_X_calib_time=new Calibration("psd0_X_calib_time.dat",16);
  Calibration *psd0_Y_calib_time=new Calibration("psd0_Y_calib_time.dat",16);
  Calibration *psd1_X_calib_time=new Calibration("psd1_X_calib_time.dat",16);
  Calibration *psd1_Y_calib_time=new Calibration("psd1_Y_calib_time.dat",16);

  Calibration *ssd_F2_calib=new Calibration("ssd_F2_calib.dat",1);
  Calibration *lowgain_calib=new Calibration("lowgain_calib.dat",2);
 



  TFile *dE_E1st = new TFile("DE_E1ST.root");
  TCutG *gate_1stTel = (TCutG*)dE_E1st->Get("DE_E1ST");
  TFile *gate_dE_E1 = new TFile("GATE_DE_E1.root");
  TCutG *gate_2ndTel = (TCutG*)gate_dE_E1->Get("GATE_DE_E1");

  TFile *gate_TDC_E1= new TFile("GATE_TDC_E1.root");
  TCutG *gate_TDC1 = (TCutG*)gate_TDC_E1->Get("GATE_TDC_E1");
  TFile *gate_TDC_E2= new TFile("GATE_TDC_E2.root");
  TCutG *gate_TDC2 = (TCutG*)gate_TDC_E2->Get("GATE_TDC_E2");
 
  if (flag_detail && flag_ssd){
    psd0_X_calib->ScanFile();
    psd0_X_calib->ShowEntries();
    psd0_Y_calib->ScanFile();
    psd0_Y_calib->ShowEntries();
    psd1_X_calib->ScanFile();
    psd1_X_calib->ShowEntries();
    psd1_Y_calib->ScanFile();
    psd1_Y_calib->ShowEntries();

    psd0_X_calib_time->ScanFile();
    psd0_X_calib_time->ShowEntries();
    psd0_Y_calib_time->ScanFile();
    psd0_Y_calib_time->ShowEntries();
    psd1_X_calib_time->ScanFile();
    psd1_X_calib_time->ShowEntries();
    psd1_Y_calib_time->ScanFile();
    psd1_Y_calib_time->ShowEntries();

    ssd_F2_calib->ScanFile();
    ssd_F2_calib->ShowEntries();

    lowgain_calib->ScanFile();
    lowgain_calib->ShowEntries();


  }
  static const UShort_t PpacSepZ = SetPpacSepZ();

//cout<<PpacSepZ<<"`````````````````````"<<endl;

  // target projection function
  // PPACa set as 0 position
  fTargetX = new TF1("fTargetX","[0]+((x/[2])*([1]-[0]))",0,1000); 
  // PPACb set as 0 -- idential results if we use the target position which subtracts [2] for calling
  //    or we could make perfectly identical results if we summed that into the function itself
  //fTargetX = new TF1("fTargetX","[0]+((x+[2])*(([1]-[0])/[2]))",0,1000); 
  fTargetX->SetParNames("PPACaX","PPACbX","PPAC Separation");
  fTargetY = new TF1("fTargetY","[0]+((x/[2])*([1]-[0]))",0,1000);
  fTargetY->SetParNames("PPACaY","PPACbY","PPAC Separation");

  const double rf=72.464; // set the RF (in ns) from 13.800 MHz cycltron readout
  const float Rfgain[2]{0.1269832,0.1268954};
//  const float Rfgain[0]=0.1269832; // RF is not converted ch into ns by rdf2root in 10Be+alpha exoeriment. Rfgain is for converting ch into ns.
//  const float Rfgain[1]=0.1268954;
  float rf_time_cal[2];//for converting Rf ch into ns
  for (int rfnum=0;rfnum<2;rfnum++){

  rf_time_cal[rfnum]=0;//initiating rf_cal
  
  }
  const double WindowZ=697.5; // distance of window position from center of PPACa
  const double cr_delay=0.;
//If the RF_vs_PPACaX histogram looks different between coin and single, you need to change cr_delay value. Read Daid's thesis.
  float tof;//TOF between PPACa and PPACb
  float rc;
//===============================================================

	Long64_t nbytes = 0, nb = 0;	

//===============================================================
   
	for(Long64_t jentry=0; jentry<nentries;jentry++) {
		
		//cout << "***********************************" << endl;
		nb = fChain->GetEntry(jentry); 
		
		if(jentry%10000 == 0)
		{
			cout << jentry << " / " << nentries << "\r" << flush;
			//cout << jentry << " / " << nentries << " Loading......." << endl;
		}
//       if (jentry > 10000000 ) break;
        


Bool_t flag_RF1=false;
Bool_t flag_RF2=false;
Bool_t flag_RF=false;
Bool_t flag_TarX=false;
Bool_t flag_TarCM=false;
Bool_t flag_Target=false;
Bool_t flag_TOF=false;
Bool_t flag_time=false;
Bool_t flag_time_target=false;
Bool_t flag_coin=false;
Bool_t flag_sing=false;
Bool_t flag_dE_E1=false;
Bool_t flag_dE_E2=false;
Bool_t flag_TTdE1=false;
Bool_t flag_TTdE2=false;
Bool_t flag_TDC1=false;
Bool_t flag_TDC2=false;
Bool_t flag_TTdET1=false;
Bool_t flag_TTdET2=false;

Bool_t flag_strange=false;//to eliminate staramge RF

if (flag_ppac){
  if(flag_raw){
/*    if (PPAC0_time[0]>0.) hPpac0TX1->Fill(PPAC0_time[0]);
    if (PPAC0_time[1]>0.) hPpac0TX2->Fill(PPAC0_time[1]);
    if (PPAC0_time[2]>0.) hPpac0TY1->Fill(PPAC0_time[2]);
    if (PPAC0_time[3]>0.) hPpac0TY2->Fill(PPAC0_time[3]);
    if (PPAC1_time[0]>0.) hPpac1TX1->Fill(PPAC1_time[0]);
    if (PPAC1_time[1]>0.) hPpac1TX2->Fill(PPAC1_time[1]);
    if (PPAC1_time[2]>0.) hPpac1TY1->Fill(PPAC1_time[2]);
    if (PPAC1_time[3]>0.) hPpac1TY2->Fill(PPAC1_time[3]);
*/
 
  } 
  if(flag_detail){
   
 
    for(int rfnum=0;rfnum<2;rfnum++)
    {
    rf_time_cal[rfnum]=rf_time[rfnum]*Rfgain[rfnum];//converting ch into ns

     }


/*	flag_strange=RfEliminate(rf_time_cal,run);
 	if(flag_strange){
	 continue;
	}        
	hRF1->Fill(rf_time_cal[0]);
        hRF2->Fill(rf_time_cal[1]);
*/
    Rfmodify(rf_time_cal,run);
    
    
    // Check if each PPAC is hit with valid event
    PpacIsHit0 = false;
    if (PPAC0_time[0]>0. && PPAC0_time[1]>0. && PPAC0_time[2]>0. && PPAC0_time[3]>0.) PpacIsHit0=true;
    PpacIsHit1 = false;
    if (PPAC1_time[0]>0. && PPAC1_time[1]>0. && PPAC1_time[2]>0. && PPAC1_time[3]>0.) PpacIsHit1=true;
 
	if(PpacIsHit0!=true || PpacIsHit1!=true)
	{
		continue;
	}


	float dummy;
	dummy=-10000;  
	if( (PPAC0_x<-30 && PPAC0_x>30) || (PPAC1_x<-30 && PPAC1_x>30)){
	PPAC0_x=dummy;
	PPAC1_x=dummy;
	}

 
	if (PpacIsHit0 && PpacIsHit1)//for ToF
       {
        
        tof=TimingPPAC(PPAC1_time)-TimingPPAC(PPAC0_time);
//	hPpacToF_a_b->Fill(tof);

	float ppac0_time,ppac1_time;
	ppac0_time=TimingPPAC(PPAC0_time);
	ppac1_time=TimingPPAC(PPAC1_time);
//	hPPAC0_test->Fill(ppac0_time);
//	hPPAC1_test->Fill(ppac1_time);



	}
    if (PpacIsHit0) hPpac0XY->Fill(PPAC0_x,PPAC0_y);
    if (PpacIsHit1) hPpac1XY->Fill(PPAC1_x,PPAC1_y);
          //if(1)
	  if (rf_time_cal[0] > 0.)
          {

	    if (PpacIsHit0 ){
                hPpac0XRF0->Fill(PPAC0_x,rf_time_cal[0]);
 //               if (cr[0]) hPpac0XRF0ssd->Fill(PPAC0_x,rf_time_cal[0]);
 //              if (cr[1]) hPpac0XRF0ds->Fill(PPAC0_x,rf_time_cal[0]);
			   
            }
	    if (PpacIsHit1 ){
 //             hPpac1XRF0->Fill(PPAC1_x,rf_time_cal[0]);
 //             if (cr[0]) hPpac1XRF0ssd->Fill(PPAC1_x,rf_time_cal[0]);
 //             if (cr[1]) hPpac1XRF0ds->Fill(PPAC1_x,rf_time_cal[0]);
            }
//		hRF1->Fill(rf_time_cal[0]);
          }
          if (rf_time_cal[1] > 0.)
          { 
	   /* if (PpacIsHit0 ){
              hPpac0XRF1->Fill(PPAC0_x,rf_time_cal[1]);
              if (cr[0]) hPpac0XRF1ssd->Fill(PPAC0_x,rf_time_cal[1]);
              if (cr[1]) hPpac0XRF1ds->Fill(PPAC0_x,rf_time_cal[1]);
            }*/
           /* if (PpacIsHit1 ){
              hPpac1XRF1->Fill(PPAC1_x,rf_time_cal[1]);
              if (cr[0]) hPpac1XRF1ssd->Fill(PPAC1_x,rf_time_cal[1]);
              if (cr[1]) hPpac1XRF1ds->Fill(PPAC1_x,rf_time_cal[1]);
            }*/
	//	hRF2->Fill(rf_time_cal[1]);
          }
  fTargetX->SetParameters(PPAC0_x,PPAC1_x,PpacSepZ);
  fTargetY->SetParameters(PPAC0_y,PPAC1_y,PpacSepZ);
  //reset
  TargetX=-100.; 
  TargetY=-100.;
              TargetX=fTargetX->Eval(WindowZ);
              TargetY=fTargetY->Eval(WindowZ); // PPACa center to window

//	hTargetXY->Fill(TargetX,TargetY);	
//        hTargetX->Fill(TargetX); 



	float xc=TargetX-4.118;
	float yc=TargetY-0.387;
	rc=sqrt(xc*xc+yc*yc);
//	hTar_Cen_Modi->Fill(rc);


  }


	if( (rf_time_cal[0]>32. && rf_time_cal[0]<40) ) flag_RF1=true;//RF1 cut for physics and beam run #31
	if( (rf_time_cal[1]>9.1 && rf_time_cal[1]<17.3) || (rf_time_cal[1]>82.1 && rf_time_cal[1]<90.3) ) flag_RF2=true;//RF2 cut for physics

//	if( (rf_time_cal[0]>19. && rf_time_cal[0]<25.) ||(rf_time_cal[0]>92. && rf_time_cal[0]<98 ) ) flag_RF1=true;//RF1 cut for beam analysis except for Run#31
//	if( (rf_time_cal[1]>67. && rf_time_cal[1]<74.) ) flag_RF2=true;//RF2 cut for beam analysis

	if (flag_RF1 || flag_RF2 ){

		flag_RF=true;
	}
/*	else{
		flag_RF=false;
	}*/

//	if(TargetX>=-5.0 && TargetX<=30.0) flag_TarX=true;
	if(rc>=0.0 && rc<=20.0) flag_TarCM=true;
//	if(flag_TarX && flag_TarCM) flag_Target=true;
	if(flag_TarCM) flag_Target=true;
	
	if(tof>3.5 && tof<5.5) flag_TOF=true;
	if(flag_RF && flag_TOF) flag_time=true;	
	if(flag_time && flag_Target) flag_time_target=true;
	if( cr[0] && !(cr[3]) )	flag_coin=true;
	if( cr[1] && !(cr[3]) ) flag_sing=true;

	//hRF1->Fill(rf_time_cal[0]);
        //hRF2->Fill(rf_time_cal[1]);


	
//F2 analysis
/////////////////////////////////////////////////////////////////////////


	SSD_F2_Calib[0]= ssd_F2_calib->Calib(0,SSD0_energy);
//	SSD_F2_Calib[0]= 0.0203259*(SSD0_energy-(-256.257779));
	hF2Energy->Fill(SSD_F2_Calib[0]);
//	hF2Energy->Fill(SSD0_energy);
	hF2Energy_RF1->Fill(rf_time_cal[0],SSD_F2_Calib[0]);
	hF2Energy_RF2->Fill(rf_time_cal[1],SSD_F2_Calib[0]);
	
////////////////////////////////////////////////////////////////////////








////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
//"continue" group except for 2 dimensional gates	
if(flag_RF!=true){
		continue;
}

if(flag_TOF!=true){	
		continue;
}
if(flag_Target!=true){
		continue;
}
//////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////
	hRF1->Fill(rf_time_cal[0]);
        hRF2->Fill(rf_time_cal[1]);

	hPpacToF_a_b->Fill(tof);
	hPPAC->Fill(PPAC0_x);


if(flag_sing){
	hPpac0XRF0->Fill(PPAC0_x,rf_time_cal[0]);
	hPpac0XRF1->Fill(PPAC0_x,rf_time_cal[1]);
	
		}



	
if(flag_coin){
	hTargetXY->Fill(TargetX,TargetY);	
        hTargetX->Fill(TargetX); 
	hTargetY->Fill(TargetY);
}



if(flag_coin)
{
}

  




}

if (flag_ssd && flag_raw){

		for(int strip=0; strip<16; strip++){
		  hSiStripE[0]->Fill(strip,PSD0_energy[strip]); //PSD1 X
		  hSiStripE[1]->Fill(strip,PSD0_energy[strip+16]); //PSD1 Y
	          hSiStripE[2]->Fill(strip,PSD1_energy[strip]); //PSD2 X
	          hSiStripE[3]->Fill(strip,PSD1_energy[strip+16]); //PSD2 Y
                  

		  strip_ch[strip]->Fill(PSD0_energy[strip]);//strips of  PSD0
		  strip_ch[strip+16]->Fill(PSD0_energy[strip+16]);//strips of  PSD0
		  strip_ch[strip+32]->Fill(PSD1_energy[strip]);//strips of  PSD0
		  strip_ch[strip+48]->Fill(PSD1_energy[strip+16]);//strips of  PSD0
			

		}
	

}
if (flag_ssd && flag_detail){
        // SKIP THE DOWNSCALED AND PILE UP
        // VERY BAD FOR CHECKING PPAC (and maybe alpha calibration data?) downscale data!
//        if (!(cr[0])) continue;






		for(int strip=0; strip<16; strip++)
		{
	          PSD0_X_Calib[strip] = psd0_X_calib->CalibMeV(strip,PSD0_energy[strip]);
	          PSD0_Y_Calib[strip] = psd0_Y_calib->CalibMeV(strip,PSD0_energy[strip+16]);
	          PSD1_X_Calib[strip] = psd1_X_calib->CalibMeV(strip,PSD1_energy[strip]);
	          PSD1_Y_Calib[strip] = psd1_Y_calib->CalibMeV(strip,PSD1_energy[strip+16]);
			//cout << "PSD0_enery=" << PSD0_energy[strip] << endl;
			//cout << "PSD0_X_Calib[" <<strip<<"]"<< PSD0_X_Calib[strip] << endl; 

		if(PSD0_X_Calib[strip]>1.5)  hSiStripEcal[0]->Fill(strip,PSD0_X_Calib[strip]); //PSD1 X
	if(PSD0_Y_Calib[strip]>1.5)	  hSiStripEcal[1]->Fill(strip,PSD0_Y_Calib[strip]); //PSD1 Y
	    if(PSD1_X_Calib[strip]>1.5)      hSiStripEcal[2]->Fill(strip,PSD1_X_Calib[strip]); //PSD2 X
	     if(PSD1_Y_Calib[strip]>1.5)     hSiStripEcal[3]->Fill(strip,PSD1_Y_Calib[strip]); //PSD2 X
			//ch_str3->Fill(strip,PSD2_Y_Calib[strip]); //PSD2 Y
				

		
		  strip_cal_ch[strip]->Fill(PSD0_X_Calib[strip]);//strips of  PSD0
		  strip_cal_ch[strip+16]->Fill(PSD0_Y_Calib[strip]);//strips of  PSD0
		  strip_cal_ch[strip+32]->Fill(PSD1_X_Calib[strip]);//strips of  PSD0
		  strip_cal_ch[strip+48]->Fill(PSD1_Y_Calib[strip]);//strips of  PSD0
		
	          PSD0_X_CalT[strip] = psd0_X_calib_time->Calib(strip,PSD0_time[strip]);
	          PSD0_Y_CalT[strip] = psd0_Y_calib_time->Calib(strip,PSD0_time[strip+16]);
	          PSD1_X_CalT[strip] = psd1_X_calib_time->Calib(strip,PSD1_time[strip]);
	          PSD1_Y_CalT[strip] = psd1_Y_calib_time->Calib(strip,PSD1_time[strip+16]);
		

      /*  if(PSD0_time[strip]>0){
		hPSD0_timetest->Fill(PSD0_X_CalT[strip]);
		hPSD0_timeRaw->Fill(PSD0_time[strip]);
	}*/
		
	}

//	if(PSD0_time[8]>0){	hPSD0_timetest->Fill(PSD0_X_CalT[8]);}
	float PsdMax[4];
/*
	PsdMax[0]...PSD1a
	PsdMax[1]...PSD1b
	PsdMax[2]...PSD2a
	PsdMax[3]...PSD2b
*/
	int numMax[4];
/*
	numMax[0]...PSD1a
	numMax[1]...PSD1b
	numMax[2]...PSD2a
	numMax[3]...PSD2b
*/


        float time_1a_Pb,time_1b_Pb,time_2a_Pb,time_2b_Pb;
	PsdMax[0]=PsdMax[1]=PsdMax[2]=PsdMax[3]=0;
        numMax[0]=numMax[1]=numMax[2]=numMax[3]=0;

		SortMax(PSD0_X_Calib,PSD0_Y_Calib,PSD1_X_Calib,PSD1_Y_Calib,PsdMax,numMax);//1st telescope	  

		time_1a_Pb=PSD0_X_CalT[numMax[0]]-TimingPPAC(PPAC1_time);	
//		  hTDC1st_Beam_alpha_a->Fill(time_1a_Pb);	
	
		time_1b_Pb=PSD0_Y_CalT[numMax[1]]-TimingPPAC(PPAC1_time);	

//		  hTDC1st_Beam_alpha_b->Fill(time_1b_Pb);	

		time_2a_Pb=PSD1_X_CalT[numMax[2]]-TimingPPAC(PPAC1_time);	
//		  hTDC2nd_Beam_alpha_a->Fill(time_2a_Pb);	
	
		time_2b_Pb=PSD1_Y_CalT[numMax[3]]-TimingPPAC(PPAC1_time);	
//		  hTDC2nd_Beam_alpha_b->Fill(time_2b_Pb);	

		E_total[0]->Fill( E_Modi(PsdMax[0],PsdMax[1],1,1,1) );

		hPSD1aMax->Fill(PsdMax[0]);
		hPSD1bMax->Fill(PsdMax[1]);

/*	if(PsdMax[0]<0.5) continue;
	if(PsdMax[1]<0.5) continue;
	if(PsdMax[2]<0.5) continue;
	if(PsdMax[3],0.5) continue;
*/


if(flag_time && flag_sing){

// if(PpacIsHit0) hPpac0XY->Fill(PPAC0_x,PPAC0_y);
// if(PpacIsHit1)	hPpac1XY->Fill(PPAC1_x,PPAC1_y);

}

if(flag_time && flag_sing){

//	hPSD1ab_maxstrip ->Fill( numMax[0] ,numMax[1]  ); 

}


if( gate_TDC1->IsInside(E_Modi(PsdMax[0],PsdMax[1],1,1,1),time_1b_Pb))    
{
	flag_TDC1=true;
}

if( gate_TDC2->IsInside(E_Modi(PsdMax[2],PsdMax[3],1,1,1),time_2b_Pb) )
{
	flag_TDC2=true;
}

if( gate_1stTel->IsInside(PsdMax[0],E_Modi(PsdMax[0],PsdMax[1],1,1,1))) 
{
	flag_dE_E1=true;
}

if(gate_2ndTel->IsInside(PsdMax[2],E_Modi(PsdMax[2],PsdMax[3],1,1,1))) 
{
	flag_dE_E2=true;
}
	if(flag_time_target && flag_dE_E1) flag_TTdE1=true;
	if(flag_time_target && flag_dE_E2) flag_TTdE2=true;
	if(flag_TTdE1 && flag_TDC1) flag_TTdET1=true;
	if(flag_TTdE2 && flag_TDC2) flag_TTdET2=true;

///////////////////////////////////////
//beam analysis

	Lowgain_Calib[0]=lowgain_calib->Calib(0,SSD1_energy);
	Lowgain_Calib[1]=lowgain_calib->Calib(1,SSD2_energy);




	if(Lowgain_Calib[0]>2.){
//	hLowgain8->Fill(Lowgain_Calib[0]);
	}

	if(Lowgain_Calib[1]>2.){
//	hLowgain9->Fill(Lowgain_Calib[1]);
	}

//	dE_E[0]->Fill( Lowgain_Calib[0],E_Modi(Lowgain_Calib[0],PsdMax[1],2,7,1));  

//	if(Lowgain_Calib[0]>1){
	hLowgain8->Fill(E_Modi(Lowgain_Calib[0],PsdMax[1],2,4,2) );
//	}

	if(Lowgain_Calib[1]>2){
//	hLowgain9->Fill(E_Modi(Lowgain_Calib[1],PsdMax[1],2,3,2) );
	}

	hCheck_E_total->Fill(E_Modi(PsdMax[0],PsdMax[1],2,7,1));

//	hCheck_E_total->Fill(PsdMax[0]);
//	hCheck_E_total_RF->Fill(PsdMax[1]);

	
	hLowgain8_raw->Fill(SSD1_energy);
	hLowgain9_raw->Fill(SSD2_energy);



//particle, pressure,gas


//	particle
//	1...alpha
//	2...10Be


//	gas
//	1...He
//	2...Ar


//	pressure...for 10Be beam.for the modification of Energy because of He between 1st layer and 2nd layer
//	if 10Be beam He run
//	1...100Torr
//	2...200Torr
//	3...300Torr
//	4...400Torr
//	5...500Torr the beam didn't reach the socond layer, so this modification is not meaningful.
//	6...550Torr
//	7...0Too		
//	if 10Be beam Ar run
//	1...25Torr
//	2...50Torr
//	3...75Torr
//	4...0Torr
//	if a resonant scattering run
//	1...700Torr
//	2...0Torr(calibration)



///////////////////////////////////////	

////////////////////////////////////////////////////
//"continue" by 2 dimensional gates
if(flag_dE_E1!=true){

//		continue;
}


if(flag_TDC1!=true){
//		continue;
}
if(flag_dE_E2!=true){

//		continue;
}


if(flag_TDC2!=true){
	//	continue;
}

/////////////////////////////////////////////
//////////////////////////////////////////////








if(flag_sing){
	hPPacToFab_vs_RF1->Fill(tof,rf_time_cal[0]);
}

/////////////////////
//check gate itself

if(flag_RF){
//	hRF1->Fill(rf_time_cal[0]);
//      hRF2->Fill(rf_time_cal[1]);
}
if(flag_TOF){
	
//	hPpacToF_a_b->Fill(tof);
//	hPPacToFab_vs_RF1->Fill(tof,rf_time_cal[0]);

}
if(flag_Target){
	hTar_Cen_Modi->Fill(rc);
//        hTargetX->Fill(TargetX); 

}
		dE_E[0]->Fill( PsdMax[0],E_Modi(PsdMax[0],PsdMax[1],1,1,1));  

if(flag_dE_E1){

//		dE_E[0]->Fill( PsdMax[0],E_Modi(PsdMax[0],PsdMax[1],1,1,1));  
		hTDC1b_Pb_vs_Etot->Fill( E_Modi(PsdMax[0],PsdMax[1],1,1,1),time_1b_Pb) ;
}


if(flag_TDC1){
	//	hTDC1b_Pb_vs_Etot->Fill( E_Modi(PsdMax[0],PsdMax[1],1,1,1),time_1b_Pb) ;
	}

if(flag_dE_E2){

		dE_E[1]->Fill( PsdMax[2],E_Modi(PsdMax[2],PsdMax[3],1,1,1));  
		hTDC2b_Pb_vs_Etot->Fill( E_Modi(PsdMax[2],PsdMax[3],1,1,1),time_2b_Pb) ;
}


if(flag_TDC1){
	//	hTDC1b_Pb_vs_Etot->Fill( E_Modi(PsdMax[0],PsdMax[1],1,1,1),time_1b_Pb) ;
	}

//if(cr[2]==true)
//{
//	hPpac0XRF0->Fill(PPAC0_x,rf_time_cal[0]);
//}
//if(cr[0]==true)
//{
//	hPpac0XRF1->Fill(PPAC0_x,rf_time_cal[0]);
//}
///////////////////////









///////////////////////////////////////////////////
//check Etotal histogram

//if(flag_RF && flag_TOF && flag_Target && flag_dE_E1 && flag_TDC1 && flag_coin){
//	hCheck_E_total->Fill(E_Modi(PsdMax[0],PsdMax[1],1,1,1));
//}
/////////////////////////////////////////////////////
/////////////////////////////////////////////////////





if(flag_TTdET1 && flag_coin)//extract scatttering
{
//	hCheck_E_total->Fill(E_Modi(PsdMax[0],PsdMax[1],1,1,1));
	//hTDC1b_Pb_vs_Etot->Fill( E_Modi(PsdMax[0],PsdMax[1],1,1,1),time_1b_Pb) ;

}
/*if(flag_sing && flag_time_target && PpacIsHit0)//extract beam
{
	hPpac0XRF0->Fill(PPAC0_x,rf_time_cal[0]);
}
if(flag_coin && flag_time_target && PpacIsHit0)
{
	hPpac0XRF1->Fill(PPAC0_x,rf_time_cal[0]);
}
*/

//hCheck_E_total->Fill(E_Modi(PsdMax[0],PsdMax[1],1,1,1));




if(flag_coin && flag_dE_E1 && flag_TDC1){
		//hLastcheck->Fill(E_Modi(PsdMax[0],PsdMax[1],1,1,1));
		}
if(flag_sing){
	//hPpac0XRF0->Fill(PPAC0_x,rf_time_cal[0]);
	if(  rf_time_cal[0]>=30 && rf_time_cal[0]<=45 ) hPpac0XRF0_cut->Fill(PPAC0_x,rf_time_cal[0]);


	hPpac0XRF1->Fill(PPAC0_x,rf_time_cal[1]);
	if(  rf_time_cal[1]>=80 && rf_time_cal[1]<=95 ) hPpac0XRF1_cut->Fill(PPAC0_x,rf_time_cal[1]);

		}





  if(flag_RF && flag_TOF)
  {
	//hCheck_E_total_RF_TOF->Fill(E_Modi(PsdMax[0],PsdMax[1],1,1,1));
  }

   if(flag_RF && flag_TOF && flag_Target)
    {
	//hCheck_E_total_RF_TOF_Target->Fill(E_Modi(PsdMax[0],PsdMax[1],1,1,1));
    }
      if(flag_RF && flag_TOF && flag_Target && flag_coin)
      {
	//hCheck_E_total_RF_TOF_Target_coin->Fill(E_Modi(PsdMax[0],PsdMax[1],1,1,1));
      }
	if(flag_RF && flag_TOF && flag_Target && flag_coin && flag_dE_E1)
	{
	//hCheck_E_total_RF_TOF_Target_coin_dEE->Fill(E_Modi(PsdMax[0],PsdMax[1],1,1,1));
	}
	  if(flag_RF && flag_TOF && flag_Target && flag_coin && flag_dE_E1 && flag_TDC1)
	    {
//		hLastcheck->Fill(E_Modi(PsdMax[0],PsdMax[1],1,1,1));
//		dE_E[0]->Fill( PsdMax[0],E_Modi(PsdMax[0],PsdMax[1],1,1,1));  
	    }
	
	hCheck_E_total_RF_TOF_Target->Fill(PsdMax[1]);
	hCheck_E_total_RF_TOF_Target_coin->Fill(PsdMax[3]);

	hCheck_E_total_RF_TOF_Target_coin_dEE->Fill(E_Modi(PsdMax[0],PsdMax[1],1,2,1));
	
	float PsdCoordinate1[3],PPACaCoordinate[3];
	PPACaCoordinate[0]=PPACaCoordinate[1]=PPACaCoordinate[2]=0;
	PsdCoordinate1[0]=PsdCoordinate1[1]=PsdCoordinate1[2]=0;
	SetCoordinate(numMax,PsdCoordinate1);
	PPACaCenter(PPACaCoordinate);
	float Ead;
	Ead=E_Modi(PsdMax[0],PsdMax[1],1,1,1);
	float Zreaction[1],ECM[1];
	Zreaction[0]=ECM[0]=0;
	FindEcm(PsdCoordinate1,Ead,Zreaction,ECM);

//	cout<<"PsdCoordinate1=	"<<PsdCoordinate1[2]<<endl;
	//cout<<"PPACa=	"<<PPACaCoordinate[1]<<endl;

	int p;
	p=0;
	float zch,dzch;

	dzch=(1227.5-692.5)/1024.;//1227.5 is PSDz. 692.5 is window z. 
	zch=692.5;

/*	for(p=0;p<=1024;p++){
	cout<<"cosA"<<cosA(zch,PsdCoordinate1)<<endl;	
	cout<<"la"<<la(zch,PsdCoordinate1)<<endl;
	cout<<"lb"<<lb(zch,PsdCoordinate1)<<endl;
	cout<<"lc"<<lc(zch,PsdCoordinate1)<<endl;

	zch+=dzch;

	}
*/	

	hZreaction->Fill(Zreaction[0]);
	hECM->Fill(ECM[0]);
	hcosA->Fill(cosA(Zreaction[0],PsdCoordinate1));
	hacosA->Fill(acos(cosA(Zreaction[0],PsdCoordinate1)));			

	float OmegaCm,OmegaLab;

	OmegaLab=50.*50./la(Zreaction[0],PsdCoordinate1)/la(Zreaction[0],PsdCoordinate1)*1000;//scale is msr	
	OmegaCm=4*cosA(Zreaction[0],PsdCoordinate1)*OmegaLab;
	hSoAVsEcm->Fill(ECM[0],OmegaCm);

	hPsd1ab_strip->Fill(numMax[0],numMax[1]);




}

		
	}

}

float TimingPPAC(Float_t PpacTime[])
{
	float TXsum,TYsum,TXaverage,TYaverage;
	TXsum=PpacTime[0]+PpacTime[1];
	TYsum=PpacTime[2]+PpacTime[3];
	TXaverage=0.5*TXsum;
	TYaverage=0.5*TYsum;
	return 0.5*(TXaverage+TYaverage);		
}

void SortMax(Double_t PsdEne1a[],Double_t PsdEne1b[],Double_t PsdEne2a[],Double_t PsdEne2b[],float PsdMax[],int numMax[])//fiunction for sorting PSD0/1_energyi
{
	numMax[0]=numMax[1]=numMax[2]=numMax[3]=0;
	float max1a=PsdEne1a[0];
	float max1b=PsdEne1b[0];
	float max2a=PsdEne2a[0];
	float max2b=PsdEne2b[0];
	for(int i=0;i<16;i++)//sort for 1st telescope
	{
		if(max1a<PsdEne1a[i])
		{
			max1a=PsdEne1a[i];
			numMax[0]=i;//Strip which gives maximum energy
		}
			PsdMax[0]=max1a;//maximum energy for the 1st layer	

		if(max1b<PsdEne1b[i])
		{
			max1b=PsdEne1b[i];
			numMax[1]=i;//Strip which gives maximum energy
		}
			PsdMax[1]=max1b;	//maximum energy for the 2nd layer

		if(max2a<PsdEne2a[i])
		{
			max2a=PsdEne2a[i];
			numMax[2]=i;//Strip which gives maximum energy
		}
			PsdMax[2]=max2a;//maximum energy for the 1st layer	

		if(max2b<PsdEne2b[i])
		{
			max2b=PsdEne2b[i];
			numMax[3]=i;//Strip which gives maximum energy
		}
			PsdMax[3]=max2b;	//maximum energy for the 2nd layer

	}
}

float E_Modi(float E1st,float E2nd,int particle,int pressure,int gas)
{
	float Etot,dummy,E2modi;
	dummy=-10000.;
	bool flag_a,flag_b,flag_H,flag_A;
	flag_a=false;
	flag_b=false;
	flag_H=false;
	flag_A=false;

	//return E1st+E2nd;


//	particle
//	1...alpha
//	2...10Be


	switch(particle)
	{
		case 1:
		flag_a=true;
		break;	

		case 2:
		flag_b=true;
		break;	
	
		default:
		cout<<"check particle character!"<<endl;
		break;
	}

//	gas
//	1...He
//	2...Ar



	switch(gas)
	{
		case 1:
		flag_H=true;
		break;	

		case 2:
		flag_A=true;
		break;	
	
		default:
		cout<<"check gas character!"<<endl;
		break;
	}


//	pressure...for 10Be beam.for the modification of Energy because of He between 1st layer and 2nd layer
//	if 10Be beam He run
//	1...100Torr
//	2...200Torr
//	3...300Torr
//	4...400Torr
//	5...500Torr
//	6...550Torr
//	7...0Too		
//	if 10Be beam Ar run
//	1...25Torr
//	2...50Torr
//	3...75Torr
//	4...0Torr
//	if a resonant scattering run
//	1...700Torr
//	2...0Torr(calibration)



//	gas… gas we used in 10Beam runs.
//	H… He
//	A… Ar

	Etot=dummy;

	if(E1st<0)
	{
		E1st=dummy;
	}
	else
		{
			Etot=E1st;
		}		

	
	
	if(E1st==dummy || E2nd < 0.08 )
	{
		E2nd=dummy;
		E2modi=E2nd;
	}
	else
	{	
	if(flag_a)//"a" means alpha
	        {
			 E2modi=E2nd+0.00791563/E2nd+-0.00108079/E2nd/E2nd+0.00240811+0.0816847*exp(0.326313-0.243917*E2nd);//for alpha
		switch(pressure)
			{
				case 1:
         		E2modi=E2modi-0.000637577/E2modi+-0.00110293/E2modi/E2modi+0.00324437+0.176893*exp(0.35498-0.240637*E2modi);
				break;
				case 2:
				break;

				default:
				cout<<"check pressure number!"<<endl;
				break;
			}

		}
	else if(flag_b)//"b" means 10Be
		{
         		E2modi=E2nd-0.000637577/E2nd+-0.00110293/E2nd/E2nd+0.00324437+0.176893*exp(0.35498-0.240637*E2nd);//for 10Be
		if(flag_H)
			{
			switch(pressure)
				{
					case 1://10Be beam 100Torr He
       					  E2modi=E2modi-0.0265577/E2modi+0.00185641/E2modi/E2modi+0.0242347+0.0596813*exp(0.347106-0.0869692*E2modi);
						break;
	
					case 2://200Torr
       		 			  E2modi=E2modi-0.0517344/E2modi+0.00359685/E2modi/E2modi+0.0494365+0.117832*exp(0.353193-0.0881791*E2modi);
						break;
					case 3://300Torr
       					  E2modi=E2modi-0.0757787/E2modi+0.00525241/E2modi/E2modi+0.0757013+0.174003*exp(0.362276-0.089598*E2modi);
						break;

					case 4://400Torr
         				E2modi=E2modi-0.0986462/E2modi+0.00683132/E2modi/E2modi+0.102837+0.227921*exp(0.373947-0.0909622*E2modi);
						break;

					case 5://500Torr
      				        E2modi=E2modi-0.120318/E2modi+0.00832507/E2modi/E2modi+0.130584+0.297052*exp(0.388787-0.0921641*E2modi);
						break;
				
					case 6://550Torr
       		  			E2modi=E2modi-0.130559/E2modi+0.00902288/E2modi/E2modi+0.144661+0.303831*exp(0.395832-0.0927058*E2modi);
				
					case 7:
					break;
			
					default:
						cout<<"Check pressure number!"<<endl;	
					break;
				}
			}
		else if(flag_A)
			{
			switch(pressure)
				{			
					case 1:
       		  			E2modi=E2modi-0.0313551/E2modi+0.00171993/E2modi/E2modi+0.824384+0.0977705*exp(0.340954-0.0939519*E2modi)+0.029578*E2modi+-5.8804e-5*E2modi*E2modi;
					break;

					case 2:
       		  			E2modi=E2modi-0.0599513/E2modi+0.00324437/E2modi/E2modi+0.824474+0.191195*exp(0.360519-0.0950212*E2modi)+0.0603711*E2modi+-0.000121488*E2modi*E2modi;


					break;


					case 3:
       		  			E2modi=E2modi-0.0857703/E2modi+0.00459017/E2modi/E2modi+0.824541+0.278098*exp(0.387446-0.0957023*E2modi)+0.00916439*E2modi+-0.00018558*E2modi*E2modi;


					break;

					case 4:
					break;
						
					default:
					break;				

				}

			}	
			
		}
	else
		{
			cout<<"dddddPlease input \"a\" or \"b\""<<endl;
			cout<<"flag_a="<<flag_a<<"flag_b="<<flag_b<<endl;
			cout<<"flag_H"<<flag_H<<"flag_A"<<flag_A<<endl;
		}

	}
	Etot=Etot+E2modi;

	return Etot; 

}

void Rfmodify(float rf_modi[],int run)
{

// You need check RF in the run#96/99
	float offset_rf;
if(run>=30 && run<=46)
{
 offset_rf=0;
}
 else if(run>=47 && run<=54)
  {
   offset_rf=8;
  }
    else if(run>=55 && run<=69)
    {
     offset_rf=-12;
    }
      else if(run>=70 && run<=88)
      {
       offset_rf=-7;
      }
	else if(run>=89 && run<=94)
	{
	offset_rf=-14;
	}

        else if(run>=90 && run<=95)
        {
         offset_rf=-14;
        }
         else if(run>=96 && run<=99)
          {
	   offset_rf=-7;
           }
            else if(run>=100 && run<=122)
            {
             offset_rf=-14;
            }
		else if(run>=123 && run<=128)
		{
		offset_rf=-14;
		}
		else 
		{
			offset_rf=-14;
		}
             
           

        

      

    
  
 		
	rf_modi[0]+=offset_rf;
	rf_modi[1]+=offset_rf;

}

bool RfEliminate(float rf_time_cal[],int run){

	bool flag_strange;
	flag_strange=false;


	switch(run){

	case 12:
	if( (rf_time_cal[0]>=24.38 && rf_time_cal[0]<=24.4) || (rf_time_cal[1]>=73.46 && rf_time_cal[1]<=73.48)){


	flag_strange=true;
	}

	break;

	case 19:
	if( (rf_time_cal[0]>=57.14 && rf_time_cal[0]<=57.15) || (rf_time_cal[1]>=105.95 && rf_time_cal[1]<=105.96)){


	flag_strange=true;
	}

	break;
	
	case 89:
	if( (rf_time_cal[0]>=0.12 && rf_time_cal[0]<=0.14) || (rf_time_cal[1]>=44.78 && rf_time_cal[1]<=44.80)){
	flag_strange=true;
	}
	break;

	case 90:
	if( (rf_time_cal[0]>=2.66 && rf_time_cal[0]<=2.68) || (rf_time_cal[1]>=51.76 && rf_time_cal[1]<=51.78)){
	flag_strange=true;
	}
	break;

	case 91:
	if( (rf_time_cal[0]>=68.56 && rf_time_cal[0]<=68.58)){
	flag_strange=true;
	}
	break;

	case 92:
	if( (rf_time_cal[0]>=72.24 && rf_time_cal[0]<=72.26)){
	flag_strange=true;
	}
	break;

	case 93:
	if( (rf_time_cal[0]>=1.0 && rf_time_cal[0]<=1.02) || (rf_time_cal[1]>=49.6 && rf_time_cal[1]<=49.62)){
	flag_strange=true;
	}
	break;



	case 94:
	if( (rf_time_cal[0]>=71.1 && rf_time_cal[0]<=71.12)){
	flag_strange=true;
	}
	break;

	case 96:
	if( (rf_time_cal[0]>=1.38 && rf_time_cal[0]<=1.40) || (rf_time_cal[1]>=34.88 && rf_time_cal[1]<=34.90)){
	flag_strange=true;
	}
	break;

	case 97:
	if( (rf_time_cal[0]>=43.41 && rf_time_cal[0]<=43.45) || (rf_time_cal[1]>=92.36 && rf_time_cal[1]<=92.38)){
	//cout<<rf_time_cal[0]<<"---------------------"<<rf_time_cal[1]<<endl;
	flag_strange=true;
	}
	break;

	case 98:
	if( (rf_time_cal[0]>=73.76 && rf_time_cal[0]<=73.78) ){

	flag_strange=true;
	}
	break;
		
	case 99:	
	if(  (rf_time_cal[1]>=46.3 && rf_time_cal[1]<=46.32)){

	flag_strange=true;
	}
	break;

	case 100:
	if(  (rf_time_cal[1]>=46.94 && rf_time_cal[1]<=46.96)){

	flag_strange=true;
	}
	break;

	case 101:
	if( (rf_time_cal[1]>=38.44 && rf_time_cal[1]<=38.46)){

	flag_strange=true;
	}
	break;

	case 102:
	if(  (rf_time_cal[1]>=45.04 && rf_time_cal[1]<=45.06)){
	flag_strange=true;
	
	}
	break;
	
	case 103:
	if( (rf_time_cal[0]>=71.6 && rf_time_cal[0]<=71.62) ){
	flag_strange=true;
	}

	break;

	case 104:
	if( (rf_time_cal[1]>=43.26 && rf_time_cal[1]<=43.28)){
	
	flag_strange=true;
	}
	break;


	case 105:
	if( (rf_time_cal[0]>=1.0 && rf_time_cal[0]<=1.02) || (rf_time_cal[1]>=48.84 && rf_time_cal[1]<=48.86)){

	flag_strange=true;
	}
	break;

	case 106:
	if( (rf_time_cal[0]>=0.12 && rf_time_cal[0]<=0.14) || (rf_time_cal[1]>=48.72 && rf_time_cal[1]<=48.74)){

	flag_strange=true;
	}
	break;

	case 107:
	if( (rf_time_cal[0]>=66.14 && rf_time_cal[0]<=66.16) || (rf_time_cal[1]>=114.84 && rf_time_cal[1]<=114.86)){

	flag_strange=true;
	}
	break;

	case 108:
	if( (rf_time_cal[1]>=24.86 && rf_time_cal[1]<=24.88)){

	flag_strange=true;
	}
	break;


	case 109:
	if( (rf_time_cal[0]>=69.2 && rf_time_cal[0]<=69.22) ){

	flag_strange=true;
	}
	break;


	case 110:
	if( (rf_time_cal[0]>=2.4 && rf_time_cal[0]<=2.42) || (rf_time_cal[1]>=51.26 && rf_time_cal[1]<=51.28)){

	flag_strange=true;
	}
	break;

	case 111:
	if( (rf_time_cal[0]>=74.4 && rf_time_cal[0]<=74.42)){

	flag_strange=true;
	}
	break;

	
	case 112:
	if( (rf_time_cal[0]>=70.34 && rf_time_cal[0]<=70.36) ){

	flag_strange=true;
	}
	break;

	case 113:
	break;


	case 114:
	if( (rf_time_cal[0]>=5.2 && rf_time_cal[0]<=5.22) || (rf_time_cal[1]>=54.3 && rf_time_cal[1]<=54.32)){

	flag_strange=true;
	}
	break;

	case 115:
	if( (rf_time_cal[0]>=48.12 && rf_time_cal[0]<=48.14) || (rf_time_cal[1]>=98.94 && rf_time_cal[1]<=98.96)){

	flag_strange=true;
	}
	break;


	case 116:
	if(  (rf_time_cal[1]>=25.36 && rf_time_cal[1]<=25.38)){

	flag_strange=true;
	}
	break;

	case 117:
	if( (rf_time_cal[0]>=0.62 && rf_time_cal[0]<=0.64) || (rf_time_cal[1]>=48.72 && rf_time_cal[1]<=48.74)){

	flag_strange=true;
	}
	break;


	case 118:
	if( (rf_time_cal[0]>=72.62 && rf_time_cal[0]<=72.64) ){

	flag_strange=true;
	}
	break;

	case 119:
	if( (rf_time_cal[1]>=43 && rf_time_cal[1]<=43.02)){

	flag_strange=true;
	}
	break;


	case 120:
	if( (rf_time_cal[0]>=77.7 && rf_time_cal[0]<=77.72) ){

	flag_strange=true;
	}
	break;

	case 121:
	if( (rf_time_cal[0]>=66.52 && rf_time_cal[0]<=66.54) || (rf_time_cal[1]>=115.22 && rf_time_cal[1]<=115.24)){

	flag_strange=true;
	}
	break;

	case 122:
	if( (rf_time_cal[0]>=68.82 && rf_time_cal[0]<=68.84) ){

	flag_strange=true;
	}
	break;

	case 123:
	if(  (rf_time_cal[1]>=26.38 && rf_time_cal[1]<=26.40)){
	flag_strange=true;
	}
	break;

	case 124:
	if(  (rf_time_cal[1]>=41.86 && rf_time_cal[1]<=41.88)){
	flag_strange=true;
	}
	break;

	case 125:
	if( (rf_time_cal[0]>=68.68 && rf_time_cal[0]<=68.70)){
	flag_strange=true;
	}
	break;

	case 126:
	if( (rf_time_cal[0]>=1.0 && rf_time_cal[0]<=1.02) || (rf_time_cal[1]>=49.22 && rf_time_cal[1]<=49.24)){
	flag_strange=true;
	}
	break;

	case 127:
	if(  (rf_time_cal[1]>=46.82 && rf_time_cal[1]<=46.84)){
	flag_strange=true;
	}
	break;

	case 128:
	if( (rf_time_cal[0]>=74.52 && rf_time_cal[0]<=74.54)){
	flag_strange=true;
	}
	break;


	case 30:
	if( (rf_time_cal[0]>=37.32 && rf_time_cal[0]<=37.34) || (rf_time_cal[1]>=88.4 && rf_time_cal[1]>=88.42) ){
	flag_strange=true;
	}

	default:
	break;

	}

	return flag_strange;
}







void ExtraporateBeam(float z, float x[]){
	float PPACaX,PPACaY,PPACbX,PPACbY;//the Beam center position at each PPAC
	float PPAC_Separation;//distance between PPACa and PPACb

	PPACaX=-8.073;
        PPACaY=-0.4767;
        PPACbX=-2.765;
        PPACbY=-0.06453;


/*	PPACaX=2.2856;
	PPACaY=-1.14;
	PPACbX=6.015;
	PPACbY=-0.155;
*/
	PPAC_Separation=299.5;

	x[0]=PPACaX+( (z/PPAC_Separation)*(PPACaX-PPACbX)    );
	x[1]=PPACaY+( (z/PPAC_Separation)*(PPACaY-PPACbY)    );

}


void SetCoordinate(int numMax[],  float PsdCoordinate1[]){

	if(numMax[0]==0){
	PsdCoordinate1[0]=-50./16./2+(-50./16.*7);/*アラインメントのレーザーがPSD1aの中心に当たっていると仮定。-50./16./2はoffsetで１つの区画あたり50./16.の長さがある。   */
	}	
	if(numMax[0]==1){
	PsdCoordinate1[0]=-50./16./2+(-50./16.*6);
		}
	if(numMax[0]==2){
	PsdCoordinate1[0]=-50./16./2+(-50./16.*5);
	}
	if(numMax[0]==3){
	PsdCoordinate1[0]=-50./16./2+(-50./16.*4);
	}
	if(numMax[0]==4){
	PsdCoordinate1[0]=-50./16./2+(-50./16.*3);
	}
	if(numMax[0]==5){
	PsdCoordinate1[0]=-50./16./2+(-50./16.*2);
	}
	if(numMax[0]==6){
	PsdCoordinate1[0]=-50./16./2+(-50./16.*1);
	}
	if(numMax[0]==7){
	PsdCoordinate1[0]=-50./16./2+(-50./16.*0);
	}
	if(numMax[0]==8){
	PsdCoordinate1[0]=-50./16./2+(-50./16.*(-1));
	}
	if(numMax[0]==9){
	PsdCoordinate1[0]=-50./16./2+(-50./16.*(-2));
	}
	if(numMax[0]==10){
	PsdCoordinate1[0]=-50./16./2+(-50./16.*(-3));
	}
	if(numMax[0]==11){
	PsdCoordinate1[0]=-50./16./2+(-50./16.*(-4));
	}
	if(numMax[0]==12){
	PsdCoordinate1[0]=-50./16./2+(-50./16.*(-5));
	}
	if(numMax[0]==13){
	PsdCoordinate1[0]=-50./16./2+(-50./16.*(-6));
	}
	if(numMax[0]==14){
	PsdCoordinate1[0]=-50./16./2+(-50./16.*(-7));
	}
	if(numMax[0]==15){
	PsdCoordinate1[0]=-50./16./2+(-50./16.*(-8));
	}
		if(numMax[1]==0){
		PsdCoordinate1[1]=-50./16./2+(-50./16.*7);/*アラインメントのレーザーがPSD1bの中心に当たっていると仮定。-50./16./2はoffsetで１つの区画あたり50./16.の長さがある。   */
		}
		if(numMax[1]==1){
		PsdCoordinate1[1]=-50./16./2+(-50./16.*6);
		}
		if(numMax[1]==2){
		PsdCoordinate1[1]=-50./16./2+(-50./16.*5);
		}
		if(numMax[1]==3){
		PsdCoordinate1[1]=-50./16./2+(-50./16.*4);
		}
		if(numMax[1]==4){
		PsdCoordinate1[1]=-50./16./2+(-50./16.*3);
		}
		if(numMax[1]==5){
		PsdCoordinate1[1]=-50./16./2+(-50./16.*2);
		}
		if(numMax[1]==6){
		PsdCoordinate1[1]=-50./16./2+(-50./16.*1);
		}
		if(numMax[1]==7){
		PsdCoordinate1[1]=-50./16./2+(-50./16.*0);
		}
		if(numMax[1]==8){
		PsdCoordinate1[1]=-50./16./2+(-50./16.*(-1));
		}
		if(numMax[1]==9){
		PsdCoordinate1[1]=-50./16./2+(-50./16.*(-2));
		}
		if(numMax[1]==10){
		PsdCoordinate1[1]=-50./16./2+(-50./16.*(-3));
		}
		if(numMax[1]==11){
		PsdCoordinate1[1]=-50./16./2+(-50./16.*(-4));
		}
		if(numMax[1]==12){
		PsdCoordinate1[1]=-50./16./2+(-50./16.*(-5));
		}
		if(numMax[1]==13){
		PsdCoordinate1[1]=-50./16./2+(-50./16.*(-6));
		}
		if(numMax[1]==14){
		PsdCoordinate1[1]=-50./16./2+(-50./16.*(-7));
		}
		if(numMax[1]==15){
		PsdCoordinate1[1]=-50./16./2+(-50./16.*(-8));
		}

	PsdCoordinate1[2]=1227.5;

		


}



float cosA(float z,float PsdCoordinate1[]){

	float PPACaCoordinate[3];
	float LA,LB,LC;
	//PSD1a、b間には7mmほど隙間があったはず。この隙を現在無視した状態。あとで補正する必要が出るかもしれない。
	PPACaCoordinate[0]=PPACaCoordinate[1]=PPACaCoordinate[2]=0;
	
	LA=LB=LC=0;
	LA=la(z,PsdCoordinate1);
	LB=lb(z,PsdCoordinate1);
	LC=lc(z,PsdCoordinate1);

	return (-1)*(  pow(LA,2)+pow(LB,2)-pow(LC,2)  )/(2*LA*LB );
	


}


float la(float z,float PsdCoordinate1[]){

	float beamx[2],PPACaCoordinate[3];
	beamx[0]=beamx[1]=0;
	
	ExtraporateBeam(z,beamx);
	PPACaCenter( PPACaCoordinate );

	return sqrt( pow(PsdCoordinate1[0]-beamx[0],2 )+pow(PsdCoordinate1[1]-beamx[1],2)+pow(PsdCoordinate1[2]-z,2)   );

}

float lb(float z,float PsdCoordinate1[]){
	float beamx[2],PPACaCoordinate[3];
	beamx[0]=beamx[1]=0;
	PPACaCoordinate[0]=PPACaCoordinate[1]=PPACaCoordinate[2]=0;
	
	ExtraporateBeam(z,beamx);
	PPACaCenter( PPACaCoordinate );

	return sqrt( pow(PPACaCoordinate[0]-beamx[0],2 )+pow(PPACaCoordinate[1]-beamx[1],2)+pow(PPACaCoordinate[2]-z,2)   );


}


float lc(float z,float PsdCoordinate1[]){
	float beamx[2],PPACaCoordinate[3];
	beamx[0]=beamx[1]=0;
	PPACaCoordinate[0]=PPACaCoordinate[1]=PPACaCoordinate[2]=0;
	
	ExtraporateBeam(z,beamx);
	PPACaCenter( PPACaCoordinate );

	return sqrt( pow(PPACaCoordinate[0]-PsdCoordinate1[0],2 )+pow(PPACaCoordinate[1]-PsdCoordinate1[1],2)+pow(PPACaCoordinate[2]-PsdCoordinate1[2],2)   );


}

void PPACaCenter(float PPACaCoordinate[]){
	
	PPACaCoordinate[0]=-8.073;
        PPACaCoordinate[1]=-0.4767;
	PPACaCoordinate[2]=0;
}



float DistanceVsE(float Ead){
	float a,b,c,d;
	a               = 8.58747   ; 
        b               = 19.0078    ;
        c               = 5.05212    ;
        d               = -0.0177869 ;

	return a+b*Ead+c*Ead*Ead+d*Ead*Ead*Ead;


}

float EVsDistance(float dis){
	float a,b,c,d,e;
	a               = -1.14326      ; 
        b               = 0.00269574   ;
        c               = -8.80737e-07 ;
        d               = 1.78614e-10  ;
        e               = 0.368521     ;

	return a+b*dis+c*dis*dis+d*dis*dis*dis+e*pow(dis,1./2.);

}


float Ear(float Ead, float la){

	float r;//Ealpha_vs_distanceのそのEalphaでの距離。これにlaを足してEarをだす。
	r=DistanceVsE(Ead);
	float l;
	l=la+r;
	return EVsDistance(l);

}

float Ecmal(float z,float PsdCoordinate1[],float Ead){

	float Earea;//reaction point でのalphaのエネルギー
	float laa;
	laa=la(z,PsdCoordinate1);
	Earea=Ear(Ead,laa);
	float Ecm;
	float m4,m10;
	m4=4.;
	m10=10.;
	Ecm=Earea*(m4+m10)/(4*m10*cosA(z,PsdCoordinate1)*cosA(z,PsdCoordinate1));
	return Ecm;	


}

float Ecmbea(float z){
	float a,b,c,d;
	a               = 7.39301      ;
        b               = -0.013384    ;
        c               = 1.56219e-05  ;
        d               = -4.34243e-08 ;
	float offset;
	offset=692.5;
	z=z-offset;

	return a+b*z+c*z*z+d*z*z*z;


}

void FindEcm(float PsdCoordinate1[],float Ead,float Zreaction[],float ECM[]){
	float z,dz;
	dz=(1227.5-692.5)/1024.;//1227.5 is PSDz. 692.5 is window z. 
	z=692.5;


	int i;
	i=0;

	float Ecmbeam,Ecmalpha;
	float MinEdif,zrea;
	MinEdif=100000;
	zrea=0;
	for(i=0;i<=1024;i++){
		Ecmbeam=Ecmbea(z);
		Ecmalpha=Ecmal(z,PsdCoordinate1,Ead);
		if( MinEdif>fabs(Ecmbeam-Ecmalpha) ){
		MinEdif=fabs(Ecmbeam-Ecmalpha);
		//cout<<"Ediff	="<<MinEdif<<endl;
		zrea=z;
		}
	z=z+dz;	
	}
	ECM[0]=(Ecmbea(zrea)+Ecmal(zrea,PsdCoordinate1,Ead))/2;
	Zreaction[0]=zrea;
	

}
