/*
 Copyright: 
 2011, 2012, 2013 daid KAHL

 This file is part of crabat

 crabat is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 crabat is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with crabat.  If not, see <http://www.gnu.org/licenses/>.
   
 In order to publish results obtained from crabat or any
 future derivative software, the authors must make reference
 to the paper describing it (in preparation): D. Kahl et al.
 ``Algorithms for Active Target Data Analysis'' 2013.  This is an 
 additional term to the license as stipulated in section 7b.
 Publication includes, but is not limited to: books, peer-
 reviewed journals, conference proceedings, annual reports, etc.
 
*/

/**
@file Analyzer_config.cxx
@author daid
@version 1.0
 * @brief Sets configurations which vary experiment-by-experiment
 * @details If we share analysis code across experiments,\n
 * it is impractical to get new updates which over-ride experiment\n
 * specific data.  But if we put these details all only in one file\n
 * it is much easier for different experiments to use the crabat code.\n
 * Handles MSTPC pad mapping.\n
 * Handles MSTPC pad gain.\n
 * Does detailed PPAC calibration.\n
 * Defines rf frequency.
@date 05 Dec 2011 16:03:48  
*/
// If run from ROOT, exit with message
#if defined (__CINT__)
{
cout << "This is C++ code, not a ROOT macro!" << endl;
cout << "To continue, run:" << endl;
cout << "make" << endl;
cout << "./run" << endl;
gApplication->Terminate();
}
#endif

#define Analyzer_config_cxx


UShort_t inline Analyzer::SetPpacSepZ()
{
  return(299.5) ; // mm 
  //#define PpacSepZ  156; // mm 
  //static const UShort_t PpacSepZ = 156; // mm 
}

// change this to pass by address
Bool_t inline Analyzer::WindowCut(Double_t x, Double_t y)
{
// modified from http://root.cern.ch/phpBB3/viewtopic.php?f=3&t=1145
  double radius = 20; // real value 20 mm...practically speaking 22 is the cut off for 29P bragg appearing 
  const Int_t n = 300; 
  Double_t winx[n+1],winy[n+1]; 
  Double_t rcut = 1.00*radius; 
  Double_t dphi = TMath::TwoPi()/n; 
  for (Int_t i=0;i<n;i++) { 
    winx[i] = rcut*TMath::Cos(i*dphi); 
    winy[i] = rcut*TMath::Sin(i*dphi); 
  } 
  winx[n] = winx[0]; winy[n] = winy[0]; 
  TCutG *window = new TCutG("window",n+1,winx,winy); 
  if (window->IsInside(x,y)) {
    delete window;
    return(true);
  }
  delete window;
  return (false);
}



