/*
 Copyright: 
 2010, 2011, 2012, 2013, 2015 daid KAHL

 This file is derived from crabat

 crabat is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 crabat is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with crabat.  If not, see <http://www.gnu.org/licenses/>.
 
*/
/**
@file run.h
@author daid
@version 1.0
@brief Global flag initalization, histogram memory control and initalization.
@details Basically allocates, initalizes and deletes memory for all histograms.
@date 09 Nov 2011 22:50:31 
*/
//Memory allocation / freeing for crabat
//This appeared to speed up processing time;
//however, no benchmarks were taken
//(previously some of this was done inline in Analyzer.cxx)
//Anyway, it's a little more modular feeling this way

#include <TGraph.h>
#include <TGraphErrors.h>
#include <TH2.h>
#include <TH3.h>


/**
@brief Prints the invocation usage to standard error.
@details The function is called whenever we get strange runtime options.\n
Prints to standard error the options accepted by the binary run.\n
*/
void Usage() {
  fprintf(stderr, "\nrun processes the data analysis\n");
  fprintf(stderr, "Must analyze at least one data type (raw and/or detailed)\n");
  fprintf(stderr, "Must analyze at least one detector type (ssd, ppac)\n");
  fprintf(stderr, "Copyright 2010-2015 daid KAHL (and others, see AUTHORS)\n");
  fprintf(stderr, "Usage: run [-R RunNumber] [-B ] [-I path] [-r ] [-d ] [-s ] [-p ]\n");
  fprintf(stderr, "Description of options:\n");
  fprintf(stderr, "  -R <int>\t: Call run on RunNumber <int> (can be entered manually at runtime also)\n");
  fprintf(stderr, "  -B\t\t: Call run in non-interactive Batch mode.  Requires -R flag.\n");
  fprintf(stderr, "  -I <string>\t: Use ROOT Tree input directory path <string> instead of reading from run.conf\n");
  fprintf(stderr, "  -r\t\t: Raw data histograms\n");
  fprintf(stderr, "  -d\t\t: Detailed data histograms\n");
  fprintf(stderr, "  -s\t\t: Analyze SSD data\n");
  fprintf(stderr, "  -p\t\t: Analyze PPAC data\n");
};


// read any arguments passed to run at runtime
// Never change these here!
Bool_t flag_Run = false; // flag_Run controls automated mode (no user input)
Bool_t flag_Batch = false; // controls Batch mode - must be used with -R flag as well
Bool_t flag_Header = false; // controls Batch mode - must be used with -R flag as well
Bool_t flag_Inpath = false; // pass the ROOT Tree Input Path (directory) rather than read from run.conf
Bool_t flag_raw=false, flag_detail=false;
Bool_t flag_ssd=false, flag_ppac=false; 

// settings for number of SSD strips and telescopes
const int num_telescopes=2;
const int num_strips=64;

//Histograms
const int num_events=400;

TH1F *hRf_ds[2]; 
TH1F *hRf_ssd[2]; 
TH1F *hRfcal[2]; 

TH1F *hSiPadHit ;

TH2F *hSiStripE[4] ;
TH2F *hSiStripEcal[4] ; 
TH2F *hSiStripEcal_PSD1ab;
TH2F *hSiStripEcal_PSD2ab;

TH2F *hSiPadE ;
TH2F *hSiEmax ;
TH2F *hSiPadEcal ;
//Each ssd channel
TH1F *strip_ch[num_strips];
TH1F *strip_ch_gated[num_strips];
TH1F *strip_cal_ch[num_strips];
TH2F *hRF0SSD1a;
TH2F *hRF0SSD1acal;
TH1F *strip_ch_gated30s[17];
TH1F *strip_ch_gated30scal[17];
TH1F *strip_ch_gated29p[17];
TH1F *strip_ch_gated17f[17];

//DeltaE-E
TH2F *dE_E[num_telescopes];
TH1F *E_total[num_telescopes];

TH1F *hPpacToF_30s ;
TH1F *hPpacToF_29p ;

TH2F *hRF0Tof ;
TH2F *hRF1Tof ;

TH1F *hPpac0TX1 ;
TH1F *hPpac0TX2 ;
TH1F *hPpac0TY1 ;
TH1F *hPpac0TY2 ;
TH1F *hPpac1TX1 ;
TH1F *hPpac1TX2 ;
TH1F *hPpac1TY1 ;

TH1F *hPpac1TY2 ;
TH1F *hPpacToF_a_b;
TH1F *hPPAC0_test ;
TH1F *hPPAC1_test ;
TH1F *hPSD0_timetest ;
TH1F *hPSD0_timeRaw ;
TH1F *hRF1;
TH1F *hRF2;
TH1F *hTargetX;
TH1F *hTargetY;
TH1F *hTar_Cen_Modi;

TH2F *hPPacToFab_vs_RF1;

TH2F *hPpac0XY ;
TH2F *hPpac0XYcut ;
TH2F *hPpac0XY_hit ;
TH2F *hPpac0XY_nohit ;
TH2F *hPpac1XY ;
TH2F *hPpac1XYcut ;
TH2F *hPpac1XY_hit ;
TH2F *hPpac1XY_nohit ;
TH2F *hTargetXY ;
TH2F *hTargetXZ ;
TH2F *hTargetXYcut ;
TH2F *hTargetXY_30s ;
TH2F *hTargetXYcut_30s ;
TH2F *hTargetXY_29p ;
TH2F *hPpac0XRF0 ;
TH2F *hPpac0XRF0_cut ;

TH2F *hPpac0XRF0_17f ;
TH2F *hPpac0XRF1_17f ;
TH2F *hPpac0XRF0_29p ;
TH2F *hPpac0XRF1_29p ;
TH2F *hPpac0XRF0_30s ;
TH2F *hPpac0XRF0_30s_ds ;
TH2F *hPpac0XRF1_30s ;
TH2F *hPpac0XRF0ds ;
TH2F *hPpac0XRF1ds ;
TH2F *hPpac0XRF0ssd ;
TH2F *hPpac0XRF1ssd ;
TH2F *hPpac1XRF0 ;
TH2F *hPpac1XRF0_cut ;

TH2F *hPpac1XRF0_29p ;
TH2F *hPpac1XRF1_29p ;
TH2F *hPpac1XRF0_30s ;
TH2F *hPpac1XRF1_30s ;
TH2F *hPpac1XRF0ds ;
TH2F *hPpac1XRF1ds ;
TH2F *hPpac1XRF0ssd ;
TH2F *hPpac1XRF1ssd ;
TH2F *hPpac0XRF1 ;

TH2F *hPpac0XRF1_cut ;

TH2F *hPpac1XRF1cut ;
TH2F *hPpac1XRF1 ;

TH2F *hTDC1a_Pb_vs_Etot;
TH2F *hTDC1b_Pb_vs_Etot;
TH2F *hTDC2a_Pb_vs_Etot;
TH2F *hTDC2b_Pb_vs_Etot;

TH2F *hPSD1ab_maxstrip;

TH1F *hTDC1st_Beam_alpha_a;
TH1F *hTDC1st_Beam_alpha_b;
TH1F *hTDC2nd_Beam_alpha_a;
TH1F *hTDC2nd_Beam_alpha_b;

TH1F *hCheck_E_total;
TH1F *hCheck_E_total_RF;
TH1F *hCheck_E_total_RF_TOF;
TH1F *hCheck_E_total_RF_TOF_Target;
TH1F *hCheck_E_total_RF_TOF_Target_coin;
TH1F *hCheck_E_total_RF_TOF_Target_coin_dEE;
TH1F *hLastcheck;
TH1F *hfinal;
TH1F *hPSD1aMax;
TH1F *hPSD1bMax;

TH1F *hF2Energy;
TH2F *hF2Energy_RF1;
TH2F *hF2Energy_RF2;

TH1F *hLowgain8;
TH1F *hLowgain9;
TH1F *hLowgain8_raw;
TH1F *hLowgain9_raw;
/*
        TH2F * ch_str   = new TH2F("ch_str","PSD1 X Channels vs Strip",40,-1,39,500,0,10);
        TH2F * ch_str2 = new TH2F("ch_str2","PSD2 X Calib Energy vs Strip",40,-1,39,500,0,15);
        TH2F * ch_str3 = new TH2F("ch_str3","PSD2 Y Calib Energy vs Strip",40,-1,39,500,0,15);
        TH2F * dE_E     = new TH2F("dE_E","dE_E",1000,0,10,2000,0,20);
*/

TH1F *hZreaction;
TH1F *hECM;
TH1F *hcosA;
TH1F *hacosA;
TH2F *hSoAVsEcm;
TH2F *hPsd1ab_strip;
TH1F *hPPAC;

/**
@brief Initalize the histograms based on which data are to be processed
@details This function must be called before any histograms are Fill'd
All memory allocation for histograms is completed previously in the run header\n
Here we define the histograms and set all relevant histogramming options.
*/
void HistInit(){ // histogram initalization
   
  char name[100];
  int xbin,ybin;
  double xmin,xmax,ymin,ymax;
  if (flag_ssd) {
  
    if (flag_raw){
      ybin=410;ymin=0.;ymax=4100.;
      xbin=16;xmin=-0.5;xmax=15.5;
      for (UShort_t i=0;i<4;i++) {
        sprintf(name,"hSiStripE%d",i);
        hSiStripE[i] = new TH2F(name,name,xbin,xmin,xmax,ybin,ymin,ymax);
        hSiStripE[i]->SetOption("COL");
      }
      xbin=4096;xmin=0;xmax=4096;
      for (UShort_t i=0;i<num_strips;i++) {
        sprintf(name,"strip_ch%d",i);
        strip_ch[i]= new TH1F(name, name, xbin,xmin,xmax);
        sprintf(name,"strip_ch_gated%d",i);
        strip_ch_gated[i]= new TH1F(name, name,xbin,xmin,xmax);
      }
    } // end if: flag_raw
  
    if (flag_detail){
      xbin=16;xmin=-0.5;xmax=15.5;
      ybin=400;ymin=1.5;ymax=7.;
      for (UShort_t i=0;i<4;i++) {
        sprintf(name,"hSiStripEcal%d",i);
        hSiStripEcal[i] = new TH2F(name,name,xbin,xmin,xmax,ybin,ymin,ymax);
        hSiStripEcal[i]->SetOption("COL");
      }
	
	hSiStripEcal_PSD1ab = new TH2F("hSiStripEcal_PSD1ab","hSiStripEca_PSD1ab",32,-0.5,31.5,ybin,ymin,ymax);
	hSiStripEcal_PSD1ab->SetOption("COL");		

	hSiStripEcal_PSD2ab = new TH2F("hSiStripEcal_PSD2ab","hSiStripEca_PSD2ab",32,-0.5,31.5,ybin,ymin,ymax);
	hSiStripEcal_PSD2ab->SetOption("COL");		
	

	
      xbin=3;xmin=-0.5;xmax=2.5;
      hSiEmax = new TH2F("hSiEmax","hSiEmax",xbin,xmin,xmax,ybin,ymin,ymax);
      hSiEmax->SetOption("COL");
      for (UShort_t i=0;i<num_strips;i++) {
        sprintf(name,"strip_cal_ch%d",i);
        strip_cal_ch[i]= new TH1F(name, name,ybin,ymin,ymax);
      }
    } // end if: flag_detail

    for (UShort_t i=0;i<num_telescopes;i++) {
      sprintf(name,"dE_E%d",i);
      dE_E[i]= new TH2F(name, name, 1000,0.,7., 1000,0.,20.);
      dE_E[i]->SetOption("COL");
      
    }


    for (UShort_t i=0;i<num_telescopes;i++){
      sprintf(name,"E_total%d",i);
      E_total[i]=new TH1F(name,name,200,0.,20.);
    }
  } // end if: flag_detail & flag_ssd

  if (flag_ppac){
  
    if(flag_raw){
      for (UShort_t i=0;i<2;i++){
        sprintf(name,"hRf_ds%d",i);
        hRf_ds[i] = new TH1F(name,name,1000,0,1000);
        sprintf(name,"hRf_ssd%d",i);
        hRf_ssd[i] = new TH1F(name,name,1000,0,1000);
      }
      xbin=600;xmin=-300.;xmax=300.;
      hPpac0TX1 = new TH1F("hPpac0TX1","PPACa X1",xbin,xmin,xmax);
      hPpac0TX1->GetXaxis()->SetTitle("Raw timing (ch)");
      hPpac0TX1->GetXaxis()->CenterTitle(true);
      hPpac0TX1->GetYaxis()->SetTitle("Counts");
      hPpac0TX1->GetYaxis()->CenterTitle(true);
      hPpac0TX2 = new TH1F("hPpac0TX2","PPACa X2",xbin,xmin,xmax);
      hPpac0TX2->GetXaxis()->SetTitle("Raw timing (ch)");
      hPpac0TX2->GetXaxis()->CenterTitle(true);
      hPpac0TX2->GetYaxis()->SetTitle("Counts");
      hPpac0TX2->GetYaxis()->CenterTitle(true);
      hPpac0TY1 = new TH1F("hPpac0TY1","PPACa Y1",xbin,xmin,xmax);
      hPpac0TY1->GetXaxis()->SetTitle("Raw timing (ch)");
      hPpac0TY1->GetXaxis()->CenterTitle(true);
      hPpac0TY1->GetYaxis()->SetTitle("Counts");
      hPpac0TY1->GetYaxis()->CenterTitle(true);
      hPpac0TY2 = new TH1F("hPpac0TY2","PPACa Y2",xbin,xmin,xmax);
      hPpac0TY2->GetXaxis()->SetTitle("Raw timing (ch)");
      hPpac0TY2->GetXaxis()->CenterTitle(true);
      hPpac0TY2->GetYaxis()->SetTitle("Counts");
      hPpac0TY2->GetYaxis()->CenterTitle(true);
      hPpac1TX1 = new TH1F("hPpac1TX1","PPACb X1",xbin,xmin,xmax);
      hPpac1TX1->GetXaxis()->SetTitle("Raw timing (ch)");
      hPpac1TX1->GetXaxis()->CenterTitle(true);
      hPpac1TX1->GetYaxis()->SetTitle("Counts");
      hPpac1TX1->GetYaxis()->CenterTitle(true);
      hPpac1TX2 = new TH1F("hPpac1TX2","PPACb X2",xbin,xmin,xmax);
      hPpac1TX2->GetXaxis()->SetTitle("Raw timing (ch)");
      hPpac1TX2->GetXaxis()->CenterTitle(true);
      hPpac1TX2->GetYaxis()->SetTitle("Counts");
      hPpac1TX2->GetYaxis()->CenterTitle(true);
      hPpac1TY1 = new TH1F("hPpac1TY1","PPACb Y1",xbin,xmin,xmax);
      hPpac1TY1->GetXaxis()->SetTitle("Raw timing (ch)");
      hPpac1TY1->GetXaxis()->CenterTitle(true);
      hPpac1TY1->GetYaxis()->SetTitle("Counts");
      hPpac1TY1->GetYaxis()->CenterTitle(true);
      hPpac1TY2 = new TH1F("hPpac1TY2","PPACb Y2",xbin,xmin,xmax);
      hPpac1TY2->GetXaxis()->SetTitle("Raw timing (ch)");
      hPpac1TY2->GetXaxis()->CenterTitle(true);
      hPpac1TY2->GetYaxis()->SetTitle("Counts");
      hPpac1TY2->GetYaxis()->CenterTitle(true);
    } // end if: flag_raw
    
    if (flag_detail){
      for (UShort_t i=0;i<2;i++){
        sprintf(name,"hRfcal%d",i);
        hRfcal[i] = new TH1F(name,name,120,0.,60.);
        hRfcal[i]->GetXaxis()->SetTitle("RF (ns)");
	hRfcal[i]->GetXaxis()->CenterTitle(true);
        hRfcal[i]->GetYaxis()->SetTitle("counts");
	hRfcal[i]->GetYaxis()->CenterTitle(true);
      }
      hRF0Tof = new TH2F("hRF0Tof","RF0 vs. PPAC Tof",300,0.,60.,200,0.,20.);
      hRF0Tof->SetOption("COL");
      hRF0Tof->GetXaxis()->SetTitle("RF (ns)");
      hRF0Tof->GetXaxis()->CenterTitle(true);
      hRF0Tof->GetYaxis()->SetTitle("Time of Flight (ns)");
      hRF0Tof->GetYaxis()->CenterTitle(true);
      
      hRF1Tof = new TH2F("hRF1Tof","RF1 vs. PPAC Tof",300,0.,60.,200,0.,20.);
      hRF1Tof->SetOption("COL");
      hRF1Tof->GetXaxis()->SetTitle("RF (ns)");
      hRF1Tof->GetXaxis()->CenterTitle(true);
      hRF1Tof->GetYaxis()->SetTitle("Time of Flight (ns)");
      hRF1Tof->GetYaxis()->CenterTitle(true);
      
      hPpac0XY = new TH2F("hPpac0XY","PPACa X vs. Y",1000,-40.,40.,1000,-40.,40.);
      hPpac0XY->SetOption("COL");
      hPpac0XY->GetXaxis()->SetTitle("X Position (mm)");
      hPpac0XY->GetXaxis()->CenterTitle(true);
      hPpac0XY->GetYaxis()->SetTitle("Y Position (mm)");
      hPpac0XY->GetYaxis()->CenterTitle(true);
      
      hPpac0XY_hit = new TH2F("hPpac0XY_hit","PPACa X vs. Y",1000,-40.,40.,1000,-40.,40.);
      hPpac0XY_hit->SetOption("COL");
      hPpac0XY_hit->GetXaxis()->SetTitle("X Position (mm)");
      hPpac0XY_hit->GetXaxis()->CenterTitle(true);
      hPpac0XY_hit->GetYaxis()->SetTitle("Y Position (mm)");
      hPpac0XY_hit->GetYaxis()->CenterTitle(true);
      
      hPpac0XY_nohit = new TH2F("hPpac0XY_nohit","PPACa X vs. Y",1000,-40.,40.,1000,-40.,40.);
      hPpac0XY_nohit->SetOption("COL");
      hPpac0XY_nohit->GetXaxis()->SetTitle("X Position (mm)");
      hPpac0XY_nohit->GetXaxis()->CenterTitle(true);
      hPpac0XY_nohit->GetYaxis()->SetTitle("Y Position (mm)");
      hPpac0XY_nohit->GetYaxis()->CenterTitle(true);
      
      hPpac0XYcut = new TH2F("hPpac0XYcut","PPACa X vs. Y",1000,-40.,40.,1000,-40.,40.);
      hPpac0XYcut->SetOption("COL");
      hPpac0XYcut->GetXaxis()->SetTitle("X Position (mm)");
      hPpac0XYcut->GetXaxis()->CenterTitle(true);
      hPpac0XYcut->GetYaxis()->SetTitle("Y Position (mm)");
      hPpac0XYcut->GetYaxis()->CenterTitle(true);
      
      hPpac1XY = new TH2F("hPpac1XY","PPACb X vs. Y",1000,-40.,40.,1000,-40.,40.);
      hPpac1XY->SetOption("COL");   
      hPpac1XY->GetXaxis()->SetTitle("X Position (mm)");
      hPpac1XY->GetXaxis()->CenterTitle(true);
      hPpac1XY->GetYaxis()->SetTitle("Y Position (mm)");
      hPpac1XY->GetYaxis()->CenterTitle(true);
      
      hPpac1XY_hit = new TH2F("hPpac1XY_hit","PPACb X vs. Y",1000,-40.,40.,1000,-40.,40.);
      hPpac1XY_hit->SetOption("COL");   
      hPpac1XY_hit->GetXaxis()->SetTitle("X Position (mm)");
      hPpac1XY_hit->GetXaxis()->CenterTitle(true);
      hPpac1XY_hit->GetYaxis()->SetTitle("Y Position (mm)");
      hPpac1XY_hit->GetYaxis()->CenterTitle(true);
      
      hPpac1XY_nohit = new TH2F("hPpac1XY_nohit","PPACb X vs. Y",1000,-40.,40.,1000,-40.,40.);
      hPpac1XY_nohit->SetOption("COL");   
      hPpac1XY_nohit->GetXaxis()->SetTitle("X Position (mm)");
      hPpac1XY_nohit->GetXaxis()->CenterTitle(true);
      hPpac1XY_nohit->GetYaxis()->SetTitle("Y Position (mm)");
      hPpac1XY_nohit->GetYaxis()->CenterTitle(true);
      
      hPpac1XYcut = new TH2F("hPpac1XYcut","PPACb X vs. Y",1000,-40.,40.,1000,-40.,40.);
      hPpac1XYcut->SetOption("COL");   
      hPpac1XYcut->GetXaxis()->SetTitle("X Position (mm)");
      hPpac1XYcut->GetXaxis()->CenterTitle(true);
      hPpac1XYcut->GetYaxis()->SetTitle("Y Position (mm)");
      hPpac1XYcut->GetYaxis()->CenterTitle(true);
      
      hTargetXY = new TH2F("hTargetXY","Target X vs. Y",1000,-40.,40.,1000,-40.,40.);
      hTargetXY->SetOption("COL");   
      hTargetXY->GetXaxis()->SetTitle("X Position (mm)");
      hTargetXY->GetXaxis()->CenterTitle(true);
      hTargetXY->GetYaxis()->SetTitle("Y Position (mm)");
      hTargetXY->GetYaxis()->CenterTitle(true);
      
      hTargetXZ = new TH2F("hTargetXZ","Target X vs. Z",1000,-40.,40.,48,-0.5,47.5);
      hTargetXZ->SetOption("COL");   
      hTargetXZ->GetXaxis()->SetTitle("X Position (mm)");
      hTargetXZ->GetXaxis()->CenterTitle(true);
      hTargetXZ->GetYaxis()->SetTitle("Pad Number");
      hTargetXZ->GetYaxis()->CenterTitle(true);
      
      hTargetXYcut = new TH2F("hTargetXYcut","Target X vs. Y window cut",1000,-40.,40.,1000,-40.,40.);
      hTargetXYcut->SetOption("COL");   
      hTargetXYcut->GetXaxis()->SetTitle("X Position (mm)");
      hTargetXYcut->GetXaxis()->CenterTitle(true);
      hTargetXYcut->GetYaxis()->SetTitle("Y Position (mm)");
      hTargetXYcut->GetYaxis()->CenterTitle(true);
      
      hTargetXY_30s = new TH2F("hTargetXY_30s","^{30}S Target X vs. Y",1000,-40.,40.,1000,-40.,40.);
      hTargetXY_30s->SetOption("COL");   
      hTargetXY_30s->GetXaxis()->SetTitle("X Position (mm)");
      hTargetXY_30s->GetXaxis()->CenterTitle(true);
      hTargetXY_30s->GetYaxis()->SetTitle("Y Position (mm)");
      hTargetXY_30s->GetYaxis()->CenterTitle(true);
      
      hTargetXYcut_30s = new TH2F("hTargetXYcut_30s","^{30}S Target X vs. Y window cut",1000,-40.,40.,1000,-40.,40.);
      hTargetXYcut_30s->SetOption("COL");   
      hTargetXYcut_30s->GetXaxis()->SetTitle("X Position (mm)");
      hTargetXYcut_30s->GetXaxis()->CenterTitle(true);
      hTargetXYcut_30s->GetYaxis()->SetTitle("Y Position (mm)");
      hTargetXYcut_30s->GetYaxis()->CenterTitle(true);
      
      hTargetXY_29p = new TH2F("hTargetXY_29p","^{29}P Target X vs. Y",1000,-40.,40.,1000,-40.,40.);
      hTargetXY_29p->SetOption("COL");   
      hTargetXY_29p->GetXaxis()->SetTitle("X Position (mm)");
      hTargetXY_29p->GetXaxis()->CenterTitle(true);
      hTargetXY_29p->GetYaxis()->SetTitle("Y Position (mm)");
      hTargetXY_29p->GetYaxis()->CenterTitle(true);
      
      hPpac0XRF0 = new TH2F("hPpac0XRF0","PPACa X vs. RF1",1000,-30.,30.,1000,0.,110.);
      hPpac0XRF0->SetOption("COL");
      hPpac0XRF0->GetXaxis()->SetTitle("X Position (mm)");
      hPpac0XRF0->GetXaxis()->CenterTitle(true);
      hPpac0XRF0->GetYaxis()->SetTitle("TOF_RF (ns)");
      hPpac0XRF0->GetYaxis()->CenterTitle(true);
      
      hPpac0XRF0_cut = new TH2F("hPpac0XRF0_cut","PPACa X vs. RF0",1000,-30.,30.,1000,0.,140.);
      hPpac0XRF0_cut->SetOption("COL");
      hPpac0XRF0_cut->GetXaxis()->SetTitle("X Position (mm)");
      hPpac0XRF0_cut->GetXaxis()->CenterTitle(true);
      hPpac0XRF0_cut->GetYaxis()->SetTitle("RF (ns)");
      hPpac0XRF0_cut->GetYaxis()->CenterTitle(true);

      hPpac0XRF1 = new TH2F("hPpac0XRF1","PPACa X vs. RF2",1000,-30.,30.,1000,0.,140.);
      hPpac0XRF1->SetOption("COL");
      hPpac0XRF1->GetXaxis()->SetTitle("X Position (mm)");
      hPpac0XRF1->GetXaxis()->CenterTitle(true);
      hPpac0XRF1->GetYaxis()->SetTitle("TOF_RF (ns)");
      hPpac0XRF1->GetYaxis()->CenterTitle(true);


      hPpac0XRF1_cut = new TH2F("hPpac0XRF1_cut","PPACa X vs. RF0",1000,-30.,30.,1000,0.,140.);
      hPpac0XRF1_cut->SetOption("COL");
      hPpac0XRF1_cut->GetXaxis()->SetTitle("X Position (mm)");
      hPpac0XRF1_cut->GetXaxis()->CenterTitle(true);
      hPpac0XRF1_cut->GetYaxis()->SetTitle("RF (ns)");
      hPpac0XRF1_cut->GetYaxis()->CenterTitle(true);



      
      hPpac1XRF0 = new TH2F("hPpac1XRF0","PPACb X vs. RF0",1000,-40.,40.,1000,0.,60.);
      hPpac1XRF0->SetOption("COL");
      hPpac1XRF0->GetXaxis()->SetTitle("X Position (mm)");
      hPpac1XRF0->GetXaxis()->CenterTitle(true);
      hPpac1XRF0->GetYaxis()->SetTitle("RF (ns)");
      hPpac1XRF0->GetYaxis()->CenterTitle(true);
      

      hPpac1XRF0_cut = new TH2F("hPpac1XRF0_cut","PPACb X vs. RF0",1000,-40.,40.,1000,0.,60.);
      hPpac1XRF0_cut->SetOption("COL");
      hPpac1XRF0_cut->GetXaxis()->SetTitle("X Position (mm)");
      hPpac1XRF0_cut->GetXaxis()->CenterTitle(true);
      hPpac1XRF0_cut->GetYaxis()->SetTitle("RF (ns)");
      hPpac1XRF0_cut->GetYaxis()->CenterTitle(true);


      hPpac0XRF0_29p = new TH2F("hPpac0XRF0_29p","PPACa X vs. RF0 29P",1000,-40.,40.,1000,0.,60.);
      hPpac0XRF0_29p->SetOption("COL");
      hPpac0XRF0_29p->GetXaxis()->SetTitle("X Position (mm)");
      hPpac0XRF0_29p->GetXaxis()->CenterTitle(true);
      hPpac0XRF0_29p->GetYaxis()->SetTitle("RF (ns)");
      hPpac0XRF0_29p->GetYaxis()->CenterTitle(true);
      
      hPpac0XRF1_29p = new TH2F("hPpac0XRF1_29p","PPACa X vs. RF1 29P",1000,-40.,40.,1000,0.,60.);
      hPpac0XRF1_29p->SetOption("COL");
      hPpac0XRF1_29p->GetXaxis()->SetTitle("X Position (mm)");
      hPpac0XRF1_29p->GetXaxis()->CenterTitle(true);
      hPpac0XRF1_29p->GetYaxis()->SetTitle("RF (ns)");
      hPpac0XRF1_29p->GetYaxis()->CenterTitle(true);
      
      hPpac0XRF0_17f = new TH2F("hPpac0XRF0_17f","PPACa X vs. RF0 17F",1000,-40.,40.,1000,0.,60.);
      hPpac0XRF0_17f->SetOption("COL");
      hPpac0XRF0_17f->GetXaxis()->SetTitle("X Position (mm)");
      hPpac0XRF0_17f->GetXaxis()->CenterTitle(true);
      hPpac0XRF0_17f->GetYaxis()->SetTitle("RF (ns)");
      hPpac0XRF0_17f->GetYaxis()->CenterTitle(true);
      
      hPpac0XRF1_17f = new TH2F("hPpac0XRF1_17f","PPACa X vs. RF1 17F",1000,-40.,40.,1000,0.,60.);
      hPpac0XRF1_17f->SetOption("COL");
      hPpac0XRF1_17f->GetXaxis()->SetTitle("X Position (mm)");
      hPpac0XRF1_17f->GetXaxis()->CenterTitle(true);
      hPpac0XRF1_17f->GetYaxis()->SetTitle("RF (ns)");
      hPpac0XRF1_17f->GetYaxis()->CenterTitle(true);
      
      hPpac0XRF0_30s = new TH2F("hPpac0XRF0_30s","PPACa X vs. RF0 30S",1000,-40.,40.,1000,0.,60.);
      hPpac0XRF0_30s->SetOption("COL");
      hPpac0XRF0_30s->GetXaxis()->SetTitle("X Position (mm)");
      hPpac0XRF0_30s->GetXaxis()->CenterTitle(true);
      hPpac0XRF0_30s->GetYaxis()->SetTitle("RF (ns)");
      hPpac0XRF0_30s->GetYaxis()->CenterTitle(true);
      
      hPpac0XRF0_30s_ds = new TH2F("hPpac0XRF0_30s_ds","PPACa X vs. RF0 30S DS",1000,-40.,40.,1000,0.,60.);
      hPpac0XRF0_30s_ds->SetOption("COL");
      hPpac0XRF0_30s_ds->GetXaxis()->SetTitle("X Position (mm)");
      hPpac0XRF0_30s_ds->GetXaxis()->CenterTitle(true);
      hPpac0XRF0_30s_ds->GetYaxis()->SetTitle("RF (ns)");
      hPpac0XRF0_30s_ds->GetYaxis()->CenterTitle(true);
      
      hPpac0XRF1_30s = new TH2F("hPpac0XRF1_30s","PPACa X vs. RF1 30S",1000,-40.,40.,1000,0.,60.);
      hPpac0XRF1_30s->SetOption("COL");
      hPpac0XRF1_30s->GetXaxis()->SetTitle("X Position (mm)");
      hPpac0XRF1_30s->GetXaxis()->CenterTitle(true);
      hPpac0XRF1_30s->GetYaxis()->SetTitle("RF (ns)");
      hPpac0XRF1_30s->GetYaxis()->CenterTitle(true);
      
      hPpac1XRF0_29p = new TH2F("hPpac1XRF0_29p","PPACb X vs. RF0 29P",1000,-40.,40.,1000,0.,60.);
      hPpac1XRF0_29p->SetOption("COL");
      hPpac1XRF0_29p->GetXaxis()->SetTitle("X Position (mm)");
      hPpac1XRF0_29p->GetXaxis()->CenterTitle(true);
      hPpac1XRF0_29p->GetYaxis()->SetTitle("RF (ns)");
      hPpac1XRF0_29p->GetYaxis()->CenterTitle(true);
      
      hPpac1XRF1_29p = new TH2F("hPpac1XRF1_29p","PPACb X vs. RF1 29P",1000,-40.,40.,1000,0.,60.);
      hPpac1XRF1_29p->SetOption("COL");
      hPpac1XRF1_29p->GetXaxis()->SetTitle("X Position (mm)");
      hPpac1XRF1_29p->GetXaxis()->CenterTitle(true);
      hPpac1XRF1_29p->GetYaxis()->SetTitle("RF (ns)");
      hPpac1XRF1_29p->GetYaxis()->CenterTitle(true);
      
      hPpac1XRF0_30s = new TH2F("hPpac1XRF0_30s","PPACb X vs. RF0 30S",1000,-40.,40.,1000,0.,60.);
      hPpac1XRF0_30s->SetOption("COL");
      hPpac1XRF0_30s->GetXaxis()->SetTitle("X Position (mm)");
      hPpac1XRF0_30s->GetXaxis()->CenterTitle(true);
      hPpac1XRF0_30s->GetYaxis()->SetTitle("RF (ns)");
      hPpac1XRF0_30s->GetYaxis()->CenterTitle(true);
      
      hPpac1XRF1_30s = new TH2F("hPpac1XRF1_30s","PPACb X vs. RF1 30S",1000,-40.,40.,1000,0.,60.);
      hPpac1XRF1_30s->SetOption("COL");
      hPpac1XRF1_30s->GetXaxis()->SetTitle("X Position (mm)");
      hPpac1XRF1_30s->GetXaxis()->CenterTitle(true);
      hPpac1XRF1_30s->GetYaxis()->SetTitle("RF (ns)");
      hPpac1XRF1_30s->GetYaxis()->CenterTitle(true);
      
      
      hPpac1XRF1cut = new TH2F("hPpac1XRF1cut","PPACb X vs. RF1",1000,-40.,40.,1000,0.,60.);
      hPpac1XRF1cut->SetOption("COL");
      hPpac1XRF1cut->GetXaxis()->SetTitle("X Position (mm)");
      hPpac1XRF1cut->GetXaxis()->CenterTitle(true);
      hPpac1XRF1cut->GetYaxis()->SetTitle("RF (ns)");
      hPpac1XRF1cut->GetYaxis()->CenterTitle(true);
      
      hPpac1XRF1 = new TH2F("hPpac1XRF1","PPACb X vs. RF1",1000,-40.,40.,1000,0.,60.);
      hPpac1XRF1->SetOption("COL");
      hPpac1XRF1->GetXaxis()->SetTitle("X Position (mm)");
      hPpac1XRF1->GetXaxis()->CenterTitle(true);
      hPpac1XRF1->GetYaxis()->SetTitle("RF (ns)");
      hPpac1XRF1->GetYaxis()->CenterTitle(true);
      
      hPpac0XRF0ds = new TH2F("hPpac0XRF0ds","PPACa X vs. RF0 downscale",1000,-40.,40.,1000,0.,60.);
      hPpac0XRF0ds->SetOption("COL");
      hPpac0XRF0ds->GetXaxis()->SetTitle("X Position (mm)");
      hPpac0XRF0ds->GetXaxis()->CenterTitle(true);
      hPpac0XRF0ds->GetYaxis()->SetTitle("RF (ns)");
      hPpac0XRF0ds->GetYaxis()->CenterTitle(true);
      
      hPpac0XRF1ds = new TH2F("hPpac0XRF1ds","PPACa X vs. RF1 downscale",1000,-40.,40.,1000,0.,60.);
      hPpac0XRF1ds->SetOption("COL");
      hPpac0XRF1ds->GetXaxis()->SetTitle("X Position (mm)");
      hPpac0XRF1ds->GetXaxis()->CenterTitle(true);
      hPpac0XRF1ds->GetYaxis()->SetTitle("RF (ns)");
      hPpac0XRF1ds->GetYaxis()->CenterTitle(true);
      
      hPpac0XRF0ssd = new TH2F("hPpac0XRF0ssd","PPACa X vs. RF0 SSD-OR",1000,-40.,40.,1000,0.,60.);
      hPpac0XRF0ssd->SetOption("COL");
      hPpac0XRF0ssd->GetXaxis()->SetTitle("X Position (mm)");
      hPpac0XRF0ssd->GetXaxis()->CenterTitle(true);
      hPpac0XRF0ssd->GetYaxis()->SetTitle("RF (ns)");
      hPpac0XRF0ssd->GetYaxis()->CenterTitle(true);
      
      hPpac0XRF1ssd = new TH2F("hPpac0XRF1ssd","PPACa X vs. RF1 SSD-OR",1000,-40.,40.,1000,0.,60.);
      hPpac0XRF1ssd->SetOption("COL");
      hPpac0XRF1ssd->GetXaxis()->SetTitle("X Position (mm)");
      hPpac0XRF1ssd->GetXaxis()->CenterTitle(true);
      hPpac0XRF1ssd->GetYaxis()->SetTitle("RF (ns)");
      hPpac0XRF1ssd->GetYaxis()->CenterTitle(true);
      
      hPpac1XRF0ds = new TH2F("hPpac1XRF0ds","PPACb X vs. RF0 downscale",1000,-40.,40.,1000,0.,60.);
      hPpac1XRF0ds->SetOption("COL");
      hPpac1XRF0ds->GetXaxis()->SetTitle("X Position (mm)");
      hPpac1XRF0ds->GetXaxis()->CenterTitle(true);
      hPpac1XRF0ds->GetYaxis()->SetTitle("RF (ns)");
      hPpac1XRF0ds->GetYaxis()->CenterTitle(true);
      
      hPpac1XRF1ds = new TH2F("hPpac1XRF1ds","PPACb X vs. RF1 downscale",1000,-40.,40.,1000,0.,60.);
      hPpac1XRF1ds->SetOption("COL");
      hPpac1XRF1ds->GetXaxis()->SetTitle("X Position (mm)");
      hPpac1XRF1ds->GetXaxis()->CenterTitle(true);
      hPpac1XRF1ds->GetYaxis()->SetTitle("RF (ns");
      hPpac1XRF1ds->GetYaxis()->CenterTitle(true);
      
      hPpac1XRF0ssd = new TH2F("hPpac1XRF0ssd","PPACb X vs. RF0 SSD-OR",1000,-40.,40.,1000,0.,60.);
      hPpac1XRF0ssd->SetOption("COL");
      hPpac1XRF0ssd->GetXaxis()->SetTitle("X Position (mm)");
      hPpac1XRF0ssd->GetXaxis()->CenterTitle(true);
      hPpac1XRF0ssd->GetYaxis()->SetTitle("RF (ns)");
      hPpac1XRF0ssd->GetYaxis()->CenterTitle(true);

      hPpac1XRF1ssd = new TH2F("hPpac1XRF1ssd","PPACb X vs. RF1 SSD-OR",1000,-40.,40.,1000,0.,60.);
      hPpac1XRF1ssd->SetOption("COL");
      hPpac1XRF1ssd->GetXaxis()->SetTitle("X Position (mm)");
      hPpac1XRF1ssd->GetXaxis()->CenterTitle(true);
      hPpac1XRF1ssd->GetYaxis()->SetTitle("RF (ns)");
      hPpac1XRF1ssd->GetYaxis()->CenterTitle(true);

      hPpacToF_30s = new TH1F("hPpacToF_30s","hPpacToF_30s",200,0.,20.);
      hPpacToF_29p = new TH1F("hPpacToF_29p","hPpacToF_29p",200,0.,20.);
      hPpacToF_a_b = new TH1F("hPpacToF_a_b","hPpacToF_a_b",200,0.,20.);

      hPPacToFab_vs_RF1 = new TH2F("hPPacToFab_vs_RF1","ToF_vs_RF1", 1000,0.,10, 1000,0.,100.);
      hPPacToFab_vs_RF1->SetOption("COL");
      hPPAC0_test = new TH1F("hPPAC0_test","hPPAC0_test",500,0.,500.);
      hPPAC1_test = new TH1F("hPPAC1_test","hPPAC1_test",500,0.,500.);
      hPSD0_timetest = new TH1F("hPSD0_timetest","hPSD0_timetest",5000,0.,5000);
      hPSD0_timeRaw = new TH1F("hPSD0_timeRaw","hPSD0_timeRaw", 5000,0.,5000);

      hRF1 = new TH1F("hRF1","hRF1",10000,0.,200.);
      hRF2 = new TH1F("hRF2","hRF2",10000,0.,200.);
      hTargetX = new TH1F("hTargetX","hTargetX",1000,-50.,50.);
      hTargetY = new TH1F("hTargetY","hTargetY",1000,-50.,50.);
      hTar_Cen_Modi= new TH1F("hTar_Cen_Modi","hTar_Cen_Modi",1000.,0.,50.);

      hTDC1st_Beam_alpha_a = new TH1F("hTDC1st_Beam_alpha_a","hTDC1st_Beam_alpha_a",1000,0.,1000.);
      hTDC1st_Beam_alpha_a->GetXaxis()->SetTitle("TDC1st_1a_from PPACb(ns)");
      hTDC1st_Beam_alpha_b = new TH1F("hTDC1st_Beam_alpha_b","hTDC1st_Beam_alpha_b",1000,0.,1000.);
      hTDC1st_Beam_alpha_b->GetXaxis()->SetTitle("TDC1st_1b_from PPACb(ns)");

      hTDC2nd_Beam_alpha_a = new TH1F("hTDC2nd_Beam_alpha_a","hTDC2nd_Beam_alpha_a",1000,0.,1000.);
      hTDC2nd_Beam_alpha_a->GetXaxis()->SetTitle("TDC2nd_1a_from PPACb(ns)");
      hTDC2nd_Beam_alpha_b = new TH1F("hTDC2nd_Beam_alpha_b","hTDC2nd_Beam_alpha_b",1000,0.,1000.);
      hTDC2nd_Beam_alpha_b->GetXaxis()->SetTitle("TDC2nd_1b_from PPACb(ns)");
     
      hTDC1a_Pb_vs_Etot = new TH2F("hTDC1a_Pb_vs_Etot","hTDC1a_Pb_vs_Etot",1000,0.,25.,1000,0.,500.);
      hTDC1a_Pb_vs_Etot->SetOption("COL");
      hTDC1b_Pb_vs_Etot = new TH2F("hTDC1b_Pb_vs_Etot","hTDC1b_Pb_vs_Etot",1000,0.,25.,1000,0.,500.);
      hTDC1b_Pb_vs_Etot->SetOption("COL");
      hTDC2a_Pb_vs_Etot = new TH2F("hTDC2a_Pb_vs_Etot","hTDC2a_Pb_vs_Etot",1000,0.,25.,1000,0.,500.);
      hTDC2a_Pb_vs_Etot->SetOption("COL");
      hTDC2b_Pb_vs_Etot = new TH2F("hTDC2b_Pb_vs_Etot","hTDC2b_Pb_vs_Etot",1000,0.,25.,1000,0.,500.);
      hTDC2b_Pb_vs_Etot->SetOption("COL");

    hCheck_E_total=new TH1F("hCheck_E_total","hCheck_E_total",1000,2.,20.);                       
    hCheck_E_total_RF=new TH1F("hCheck_E_total_RF","hCheck_E_total_RF",1000,2.,20.);
    hCheck_E_total_RF_TOF=new TH1F("hCheck_E_total_RF_TOF","hCheck_E_total_RF_TOF",200,4.,20.);
    hCheck_E_total_RF_TOF_Target=new TH1F("hCheck_E_total_RF_TOF_Target","hCheck_E_total_RF_TOF_Target",1000,0.,18.);
    hCheck_E_total_RF_TOF_Target_coin=new TH1F("hCheck_E_total_RF_TOF_Target_coin","hCheck_E_total_RF_TOF_Target_coin",1000,0.,18.);    
    hCheck_E_total_RF_TOF_Target_coin_dEE=new TH1F("hCheck_E_total_RF_TOF_Target_coin_dEE","hCheck_E_total_RF_TOF_Target_coin_dEE",1000,0.,18.);
    hLastcheck = new TH1F("hLastcheck","hLastcheck",200,4.,20.);
    hfinal = new TH1F("final","final",200,0.,20.);
    
    hPSD1ab_maxstrip = new TH2F("PSD1ab_maxstrip","PSD1ab_maxstrip", 16,0.5,16.5, 16,0.5,16.5);
    hPSD1ab_maxstrip ->SetOption("COL");

    hPSD1aMax = new TH1F("PSD1aMax","PSD1aMax", 200,0.,20.); 
    hPSD1bMax = new TH1F("PSD1bMax","PSD1bMax", 200,0.,20.);
	
    hF2Energy = new TH1F("hF2Energy","hF2Energy", 1000,0.,80.);
    hF2Energy_RF1 = new TH2F("F2Energy_vs_RF1","F2Energy_vs_RF1", 1000,0.,100, 1000,0.,80.);

	hF2Energy_RF1 ->SetOption("COL");
	hF2Energy_RF1 ->GetXaxis()->SetTitle("RF1 (ns)");
	hF2Energy_RF1 ->GetXaxis()->CenterTitle(true);
	hF2Energy_RF1 ->GetYaxis()->SetTitle("F2_Energy (MeV)");
	hF2Energy_RF1 ->GetYaxis()->CenterTitle(true);

    hF2Energy_RF2 = new TH2F("F2Energy_vs_RF2","F2Energy_vs_RF2", 1000,0.,100, 1000,0.,80.);
	   
	hF2Energy_RF2 ->SetOption("COL");
	hF2Energy_RF2 ->GetXaxis()->SetTitle("RF2 (ns)");
	hF2Energy_RF2 ->GetXaxis()->CenterTitle(true);
	hF2Energy_RF2 ->GetYaxis()->SetTitle("F2_Energy (MeV)");
	hF2Energy_RF2 ->GetYaxis()->CenterTitle(true);


     hLowgain8 = new TH1F("hLowgain8","hLowgain8", 1000,0.,30.);
     hLowgain9 = new TH1F("hLowgain9","hLowgain9", 1000,0.,30.);
     hLowgain8_raw = new TH1F("hLowgain8_raw","hLowgain8_raw", 10000,0.,4096.);
     hLowgain9_raw = new TH1F("hLowgain9_raw","hLowgain9_raw", 10000,0.,4096.);

     hZreaction = new TH1F("hZreaction","hZreaction",10000,0.,1227.5);
     hECM = new TH1F("hECM","hECM",350,1.,8.);
     hcosA = new TH1F("hcosA","hcosA", 100,0.,1.0); 
     hacosA = new TH1F("hacosA","hacosA", 1000,0.,90.); 
 
    hSoAVsEcm = new TH2F("hSoAVsEcm","hSoAVsEcm", 1000,1.,8., 10000,0.,450);
	hSoAVsEcm ->SetOption("COL");
	hSoAVsEcm ->GetXaxis()->SetTitle("Center of Mass Energy (MeV)");
	hSoAVsEcm ->GetXaxis()->CenterTitle(true);
	hSoAVsEcm ->GetYaxis()->SetTitle("Solid Angle(msr)");
	hSoAVsEcm ->GetYaxis()->CenterTitle(true);

    hPsd1ab_strip = new TH2F("hPsd1ab_strip","hPsd1ab_strip",16,-0.5,15.5, 16,-0.5,15.5);

	hPsd1ab_strip ->SetOption("COL");
	 hPsd1ab_strip->GetXaxis()->SetTitle("Psd1a)");
	 hPsd1ab_strip->GetXaxis()->CenterTitle(true);
	 hPsd1ab_strip->GetYaxis()->SetTitle("Psd1b)");
	 hPsd1ab_strip->GetYaxis()->CenterTitle(true);

    hPPAC = new TH1F("hPPAC","hPPAC", 1000,-50.,50.);



   } // end if: flag_detail
        
  } // end if: flag_ppac
  
} // close HistInit

/**
@brief Write all the histograms.
@details This function should be called after all histograms have been Fill'd in Analyzer.\n
It will write all histograms that were initalized based on the data options.\n
*/
void HistWrite() {
  // WRITE THE HISTOGRAMS

  if (flag_ssd) { // process SSD?
    if (flag_raw){ // write the SSD strip raw histograms
      for (int i=0;i<4;i++){
        hSiStripE[i]->Write(); 
      }
      for (UShort_t i=0;i<num_strips;i++) {
         strip_ch[i]->Write();
         strip_ch_gated[i]->Write();
       }
    } // end if: flag_raw
    
    if (flag_detail){ // write the SSD strip detailed histograms
      hSiEmax->Write(); 
      for (int i=0;i<4;i++){
        hSiStripEcal[i]->Write(); 
      }
	hSiStripEcal_PSD1ab->Write();
	hSiStripEcal_PSD2ab->Write();
      for (UShort_t i=0;i<num_strips;i++) {
         strip_cal_ch[i]->Write();
      }
      for (UShort_t i=0;i<num_telescopes;i++) {
         dE_E[i]->Write();
         E_total[i]->Write();
      }
    } // end if : flag_detail
  } //end if: flag_ssd

  if (flag_ppac){ // process PPAC?
    if (flag_raw) { // write the PPAC raw histograms
       for (UShort_t i=0;i<2;i++) hRf_ssd[i]->Write();
       for (UShort_t i=0;i<2;i++) hRf_ds[i]->Write();
       hPpac0TX1->Write();
       hPpac0TX2->Write();
       hPpac0TY1->Write();
       hPpac0TY2->Write();
       hPpac1TX1->Write();
       hPpac1TX2->Write();
       hPpac1TY1->Write();
       hPpac1TY2->Write();
    } // end if: flag_raw

    if (flag_detail) { // write the PPAC detailed histograms
       for (UShort_t i=0;i<2;i++) hRfcal[i]->Write();
       hRF0Tof->Write();
       hRF1Tof->Write();
       hPpac0XY->Write();
       hPpac0XY_hit->Write();
       hPpac0XY_nohit->Write();
       hPpac0XYcut->Write();
       hPpac1XY->Write();  
       hPpac1XYcut->Write();  
       hPpac1XY_hit->Write();  
       hPpac1XY_nohit->Write();  
       hTargetXY->Write();  
       hTargetXZ->Write();  
       hTargetXYcut->Write();  
       hTargetXY_30s->Write();  
       hTargetXYcut_30s->Write();  
       hTargetXY_29p->Write();  
       hPpac0XRF0->Write(); 
       hPpac0XRF0_cut->Write(); 
       hPpac1XRF0->Write(); 
       hPpac1XRF0_cut->Write(); 

       hPpac0XRF1->Write(); 
       hPpac0XRF1_cut->Write(); 
       hPpac1XRF1cut->Write(); 
       hPpac1XRF1->Write();
       hPpac0XRF0_17f->Write(); 
       hPpac0XRF1_17f->Write(); 
       hPpac0XRF0_29p->Write(); 
       hPpac0XRF1_29p->Write(); 
       hPpac0XRF0_30s->Write(); 
       hPpac0XRF0_30s_ds->Write(); 
       hPpac0XRF1_30s->Write(); 
       hPpac1XRF0_29p->Write(); 
       hPpac1XRF1_29p->Write(); 
       hPpac1XRF0_30s->Write(); 
       hPpac1XRF1_30s->Write(); 
       hPpac0XRF0ds->Write(); 
       hPpac0XRF1ds->Write(); 
       hPpac0XRF0ssd->Write(); 
       hPpac0XRF1ssd->Write(); 
       hPpac1XRF0ds->Write(); 
       hPpac1XRF1ds->Write(); 
       hPpac1XRF0ssd->Write(); 
       hPpac1XRF1ssd->Write(); 
       hPpacToF_30s->Write();
       hPpacToF_29p->Write();
       hPpacToF_a_b->Write();
       hPPacToFab_vs_RF1->Write();
       hPPAC0_test->Write();
       hPPAC1_test->Write();
       hPSD0_timetest->Write();
       hPSD0_timeRaw->Write();
       hRF1->Write();
       hRF2->Write();
       hTargetX->Write();
       hTargetY->Write();
       hTar_Cen_Modi->Write();

       hTDC1st_Beam_alpha_a->Write();
       hTDC1st_Beam_alpha_b->Write();
       hTDC2nd_Beam_alpha_a->Write();
       hTDC2nd_Beam_alpha_b->Write();
       hTDC1a_Pb_vs_Etot->Write();
       hTDC1b_Pb_vs_Etot->Write();
       hTDC2a_Pb_vs_Etot->Write();
       hTDC2b_Pb_vs_Etot->Write();

	hCheck_E_total				->Write();
	hCheck_E_total_RF			->Write();
	hCheck_E_total_RF_TOF			->Write();
	hCheck_E_total_RF_TOF_Target		->Write();
	hCheck_E_total_RF_TOF_Target_coin	->Write();
	hCheck_E_total_RF_TOF_Target_coin_dEE	->Write();
	hLastcheck				->Write();
	hfinal					->Write();
        hPSD1ab_maxstrip->Write();

	hPSD1aMax->Write();
	hPSD1bMax->Write();

	hF2Energy->Write();	
	hF2Energy_RF1->Write();
	hF2Energy_RF2->Write();

	
	hLowgain8->Write();
        hLowgain9->Write();
	hLowgain8_raw->Write();
        hLowgain9_raw->Write();

	hZreaction->Write();
        hECM->Write();
	hcosA->Write();
	hacosA->Write();
	hSoAVsEcm->Write();
	hPsd1ab_strip->Write();
	hPPAC->Write();

    } // end if: flag_detail
  } // end if: flag_ppac

} // close HistWrite


/**
@brief delete all the histograms
@details This function should be called after all histograms have been Write'n.\n
In principle it is not totally necessary to do this kind of garbage collection.\n
However, for memory checking with valgrind, this is helpful.
*/
void HistClean(){ // function to free histogram memory
  delete hSiPadHit ;
  for (int i=0;i<4;i++){
    delete hSiStripE[i] ;
    delete hSiStripEcal[i] ;
  }
  delete hSiStripEcal_PSD1ab;
  delete hSiStripEcal_PSD2ab;
  delete hSiPadE ;
  delete hSiEmax ;
  delete hSiPadEcal ;
  delete hRF0Tof;
  delete hRF1Tof;
  delete hPpac0TX1;
  delete hPpac0TX2;
  delete hPpac0TY1;
  delete hPpac0TY2;
  delete hPpac1TX1;
  delete hPpac1TX2;
  delete hPpac1TY1;
  delete hPpac1TY2;
  delete hPpac0XY ;
  delete hPpac0XY_hit ;
  delete hPpac0XY_nohit ;
  delete hPpac0XYcut ;
  delete hPpac1XY ;
  delete hPpac1XY_hit ;
  delete hPpac1XY_nohit ;
  delete hPpac1XYcut ;
  delete hTargetXY ;
  delete hTargetXZ ;
  delete hTargetXYcut ;
  delete hTargetXY_29p ;
  delete hTargetXY_30s ;
  delete hTargetXYcut_30s ;
  delete hPpac0XRF0 ;
  delete hPpac0XRF0_cut ;
  delete hPpac0XRF0_17f ;
  delete hPpac0XRF1_17f ;
  delete hPpac0XRF0_29p ;
  delete hPpac0XRF1_29p ;
  delete hPpac0XRF0_30s ;
  delete hPpac0XRF0_30s_ds ;
  delete hPpac0XRF1_30s ;
  delete hPpac0XRF0ds ;
  delete hPpac0XRF1ds ;
  delete hPpac0XRF0ssd ;
  delete hPpac0XRF1ssd ;
  delete hPpac1XRF0 ;
  delete hPpac1XRF0_cut ;
  delete hPpac1XRF0_29p ;
  delete hPpac1XRF1_29p ;
  delete hPpac1XRF0_30s ;
  delete hPpac1XRF1_30s ;
  delete hPpac1XRF0ds ;
  delete hPpac1XRF1ds ;
  delete hPpac1XRF0ssd ;
  delete hPpac1XRF1ssd ;
  delete hPpac0XRF1 ;
  delete hPpac0XRF1_cut ;
  delete hPpac1XRF1cut ;
  delete hPpac1XRF1 ;
  for (int i=0;i<2;i++) delete hRf_ds[i]; 
  for (int i=0;i<2;i++) delete hRf_ssd[i]; 
  for (int i=0;i<2;i++) delete hRfcal[i]; 
  for (int i=0;i<num_strips;i++){
    delete strip_ch[i];
    delete strip_ch_gated[i];
    delete strip_cal_ch[i];
  }
  delete hRF0SSD1a;
  delete hRF0SSD1acal;
  for (int i=0;i<17;i++){
    delete strip_ch_gated30s[i];
    delete strip_ch_gated30scal[i];
    delete strip_ch_gated29p[i];
    delete strip_ch_gated17f[i];
  }
  delete hPpacToF_30s;
  delete hPpacToF_29p;
  delete hPpacToF_a_b;
  delete hPPacToFab_vs_RF1;
  delete hPPAC0_test;
  delete hPPAC1_test;
  delete hPSD0_timetest;
  delete hPSD0_timeRaw;
  delete hRF1;
  delete hRF2;
  delete hTargetX;
  delete hTargetY;
  delete hTar_Cen_Modi;

  delete hTDC1st_Beam_alpha_a;
  delete hTDC1st_Beam_alpha_b;
  delete hTDC2nd_Beam_alpha_a;
  delete hTDC2nd_Beam_alpha_b;
  delete hTDC1a_Pb_vs_Etot;
  delete hTDC1b_Pb_vs_Etot;
  delete hTDC2a_Pb_vs_Etot;
  delete hTDC2b_Pb_vs_Etot;
  for (int i=0;i<num_telescopes;i++){
    delete dE_E[i];
    delete E_total[i];
  }
	  delete hCheck_E_total			     	;
	  delete hCheck_E_total_RF		 	;    
	  delete hCheck_E_total_RF_TOF		     	;
	  delete hCheck_E_total_RF_TOF_Target	     	;
	  delete hCheck_E_total_RF_TOF_Target_coin    	;
	  delete hCheck_E_total_RF_TOF_Target_coin_dEE	;
	  delete hLastcheck				;
	delete hfinal					;
	  delete hPSD1ab_maxstrip;  

	  delete hPSD1aMax;
	  delete hPSD1bMax;

	  delete hF2Energy;
	  delete hF2Energy_RF1;
	  delete hF2Energy_RF2;


	delete hLowgain8;
	delete hLowgain9;
	delete hLowgain8_raw; 
	delete hLowgain9_raw;
	delete hZreaction;
        delete hECM;
	delete hcosA;
	delete hacosA;
	delete hSoAVsEcm;
	delete hPsd1ab_strip;
	delete hPPAC;	
} // close HistClean    
